# -*- coding: utf-8 -*-
"""The module 'epyunit.SystemCalls' provides the controlled execution of subprocesses as units.

The features are foreseen to support easy setups for unit and regression 
tests on complex command line options and their combinations. 
These are in designed to be used in conjunction with
with PyUnit, either within Eclipse and PyDev, or from the command line.

A quick example call is::

  python -c 'from epyunit.SystemCalls import SystemCalls;x=SystemCalls(**{"proceed":"trace"});x.callit("myscript.sh xOK")'

or::

  python -c 'from epyunit.SystemCalls import SystemCalls;x=SystemCalls(**{"proceed":"trace"});x.callit("myscript.py xOK")'

**SECURITY REMARK**: Current version supports for subprocesses the ShellMode only.
For test environments in R&D this - hopefully - is perfectly OK, 
else eventual security issues has to be considered.

"""
from __future__ import absolute_import

__author__ = 'Arno-Can Uestuensoez'
__license__ = "Artistic-License-2.0 + Forced-Fairplay-Constraints"
__copyright__ = "Copyright (C) 2010-2016 Arno-Can Uestuensoez @Ingenieurbuero Arno-Can Uestuensoez"
__version__ = '0.1.12'
__uuid__='9de52399-7752-4633-9fdc-66c87a9200b8'

__docformat__ = "restructuredtext en"

import sys,datetime
version = '{0}.{1}'.format(*sys.version_info[:2])
if version < '2.7': # pragma: no cover
    raise Exception("Requires Python-2.7.* or higher")

import subprocess

# output enums
_O_STD = 1 #: output to STDOUT
_O_ERR = 2 #: output to STDERR
_O_STR = 3 #: output to STRING

class SystemCallsException(Exception):
    """Common error within epyunit.SystemCalls. 
    """
    pass

#class SystemCallsExceptionSubprocessError(SystemCallsException):

class SystemCallsExceptionSubprocessError(SystemExit):
    """Error from subprocess. 
    """
    def __init__(self,*args,**kargs):
        """Calls the 'exceptions.SystemExit' with pass through of parameters.
    
        Args:
            **kargs: Additional parameters specific for ePyUnit 
                where the interface is not actually clear.
                These are removed before the pass-through call.

                exitval: Replaces 'exceptions.SystemExit.code', the
                    first param value when of type 'int'. The default
                    is defined as '1'.
                    
                    REMARK: Did not found an official interface for
                        'exceptions.SystemExit.__init__', thus opted
                        to temporary injection.

                exitmsg: Defines display test.

        Returns:
            None/Itself

        Raises:
            itself
        
        """
        _code = kargs.get('exitval')
        if _code:
            kargs.pop('exitval')
        _msg = kargs.get('exitmsg')
        if _msg:
            kargs.pop('exitmsg')
        super(SystemCallsExceptionSubprocessError,self).__init__(*args,**kargs)
        if _code:
            self.code = _code
        if _msg:
            self.message = _msg
        pass

class SystemCalls(object):
    """Wraps system calls for subprocesses.
    
    For supported parameters refer to 'epyunit.SystemCalls.setkargs()'.
    
    """
    def __init__(self,**kargs):
        """Prepares the interface for subprocess calls with output cache.
        
        The initial setup includes the preparation of the result 
        cache for the response data from stdout and stderr.

        Args:
            **kargs: Parameters specific for the operations,
                passed through to **setkargs**.

        Returns:
            When successful returns 'True', else returns either 'False', or
            raises an exception.
            Success is the complete addition only, thus one failure returns
            False.

            *REMARK*
                When errors occur in buffered mode the
                output from the called process is possibly
                not completely cached. In that case
                switch to 'dialogue' mode in order to get an 
                unbuffered console display.

        Raises:
            passed through exceptions:
        
        """
        # initial defaults
        self.console = 'cli' #: console type for output
        self.bufsize = 16384 #: size of cache for output received from subprocess 
        #self.bufsize=-1
        
        self.emptyiserr = False 
        """The subprocess call for an empty string is 'success', 
        thus this has to be checked independently for eventually erroneously missing call string
        """
        
        self.errasexcept = False #: raises in case of errors an exception
        self.myexe = self._mode_batch #: preconfigured call back for actual execution
        self.out = 'pass' #: output format/stream
        self.outtarget = 'stdout' #: output format/stream
        self.passerr = False
        self.proceed = 'doit' #: what to do...
        self.raw = False
        self.useexit = True
        self.usestderr = False
        self.verbose = 0

        self._appname = None 
        self._testid = None 
        self._timestamp = None 
        self._environment = None 

        self.setkargs(**kargs)
        pass

    def callit(self,callstr,**kargs):
        """Executes a prepared callstr by the preset function pointer 'self.myexe'.
        
        Args:
            callstr: a prepared shell-style call.
        
            **kargs: kargs is passed through, for values refer
                to **setkargs**.

                debug: Sets debug for rule data flow.

                env: Passed through.

                proceed: Changes predefined value and
                    dispatches to the subcall.

                raw: Suppress the split of lines for
                    stdout and stderr.

                verbose: Sets verbose for rule data flow.

        Returns:
            Result of call, the format is:
            
                ret[0]::= exit value
                
                ret[1]::= STDOUT as non-processed string.
                
                ret[2]::= STDERR as non-processed string.

            *REMARK*
                When errors occur in buffered mode the
                output from the called process is possibly
                not completely cached, e.g. in case of exeptions. 
                In that case try switching to 'dialogue' mode in order
                to get an unbuffered console display. This will deliver
                - hopefully - the complete output.

        Raises:
            passed through exceptions:
            
        """
        proceed = self.proceed
        _raw = False
        for k,v in kargs.iteritems():
            if k=='proceed':
                proceed=self.get_proceed(v)
                kargs.pop('proceed')
            elif k=='raw':
                _raw=True
                kargs.pop('raw')
        self.setkargs(**kargs)
               
        ret=[1,]
        if proceed=="print":
            print str(callstr)
            return [0,[],[]]

        elif proceed=="trace":
            print >>sys.stderr, "TRACE:"+str(callstr)
            ret=self.myexe(callstr,**kargs)

        elif proceed=="doit":
            ret=self.myexe(callstr,**kargs)

        if ret[0] not in (0,5):
            if self.verbose>0:
                print "ret:"+str(ret)+" =>call:"+str(callstr)

        if not _raw and not self.raw:
            ret = self.splitLines(ret)
        return ret

    def displayit(self,ret,**kargs):
        """Displays result list ret in selected format.

        Args:
            ret: Data to be displayed.
            
            **kargs:

                outtarget: The target of display:
                
                    str: return as formatted string
                
                    stdout: print to sys.stdout
                
                    stderr: print to sys.stderr
                
                out: Output for display, valid:
                    
                    csv   : CSV with sparse records
                    
                    pass  : pass through STDOUT and STDERR from subprocess

                    repr  : Python 'repr()'
                    
                    str   : Python 'str()'
                    
                    xml   : XML
                
                default:=self.out

        Returns:
            When successful returns

                outtarget=='str': returns a printable formatted string

                outtarget in ('stdout', 'stderr',): 

                    'True':  for success

                    'False': for failure

        Raises:
            passed through exceptions:
        
        """
        _target = kargs.get('outtarget',self.outtarget)
        if _target == 'str':
            _t = _O_STR
        elif _target == 'stderr':
            _t = _O_ERR
        else:
            _t = _O_STD
        self._oc = ""

        _out = kargs.get('out',self.out)
 
        def _output(s,_o=None):
            if not _o:
                _o = _t 
            if _o == _O_STR:
                self._oc += s
            elif _o == _O_ERR:
                sys.stderr.write(s)
            else:
                sys.stdout.write(s)
             
            
        if _out == "pass": # pass through STDOUT and STDERR from subprocess
            if ret[1]:
                if type(ret[1]) == list:
                    _output("\n".join(ret[1])+'\n')
                else:
                    _output(ret[1])
            if ret[2]:
                if _t == _O_STD:
                    _tx = _O_ERR
                else:
                    _tx = _t
                if type(ret[2]) == list:
                    _output("\n".join(ret[2])+'\n',_tx)
                else:
                    _output(ret[2],_tx)

        elif _out == "repr": # Python 'repr()'
            _output(repr(ret))
        
        elif _out == "str": # Python 'str()'
            _output("exit:   "+str(ret[0])+'\n')
            _output("stdout: "+str(ret[1])+'\n')
            _output("stderr: "+str(ret[2])+'\n')
    
        elif _out == "xml": # XML
            _output("""<?xml version="1.0" encoding="UTF-8"?>\n""")
    
            # take timestamp        
            _dn = datetime.datetime.now()
            _date=str(_dn.year)+'-'+str(_dn.month)+'-'+str(_dn.day)
            _time=str(_dn.hour)+':'+str(_dn.minute)+':'+str(_dn.second)
            _head="<test-result "
            if self._testid:
                _head +=  " id="+str(self._testid)
            if self._appname:
                _head +=  " appname='"+str(self._appname)+"'"
            if self._timestamp:
                _head +=  " date='"+str(_date)+"'"
                _head +=  " time='"+str(_time)+"'"
            if self._environment:
                _head +=  " host='"+str(self._host)+"'"
                _head +=  " user='"+str(self._user)+"'"
                _head +=  " os='"+str(self._os)+"'"
                _head +=  " osver='"+str(self._osver)+"'"
                _head +=  " dist='"+str(self._dist)+"'"
                _head +=  " distver='"+str(self._distver)+"'"
            _head += ">"
    
            _output(str(_head)+'\n')
    
            # exit code
            _output("    <exit-code>"+str(ret[0])+"</exit-code>\n")
            
            # STDOUT
            _output("    <stdout>\n")
            lx = 0
            for l in ret[1]:
                _output("        <line cnt="+str(lx)+">"+str(l)+"</line>\n")
                lx += 1
            _output("    </stdout>\n")
            
            # STDERR
            _output("    <stderr>\n")
            lx = 0
            for l in ret[2]:
                _output("        <line cnt="+str(lx)+">"+str(l)+"</line>\n")
                lx += 1
            _output("    </stderr>\n")
    
            _output("""</test-result>""")
    
        elif _out == "csv": # CSV with sparse records
            _head = None
                
            if self._testid:
                if _head: _head = ';' 
                _head += "testid"
            if self._appname:
                if _head: _head = ';' 
                _head +=  ";appname"
            if self._timestamp:
                if _head: _head = ';' 
                _head += ";date;time"
            if self._environment:
                if _head: _head = ';' 
                _head += ";host;user;os;osver;dist;distver"

            if _head: _head = ';'
            else: _head = "" 
            _head += "exitcode;total-lines;stdout-line;stdout;stderr-line;stderr"
    
            _output(str(_head)+'\n')
            
            # take timestamp        
            _dn = datetime.datetime.now()
            _date=str(_dn.year)+'-'+str(_dn.month)+'-'+str(_dn.day)
            _time=str(_dn.hour)+':'+str(_dn.minute)+':'+str(_dn.second)
    
            _lxtot=0

            # output record prefix
            _recpre = None

            # common prefix
            if self._testid: 
                if _recpre: _recpre += ';'
                _recpre += str(self._testid)
            if self._appname: 
                if _recpre: _recpre += ';'
                _recpre +=  str(self._appname)
            if self._timestamp: 
                if _recpre: _recpre += ';'
                _recpre += str(_date)+";"+str(_time)
            if self._environment: 
                if _recpre: _recpre += ';'
                _recpre += str(self._host)+";"+str(self._user)+";"+str(self._os)+";"+str(self._osver)+";"+str(self._dist)+";"+str(self._distver)

            _lx1 = 0
            _lx2 = 0

            _r1 = len(ret[1])
            _r2 = len(ret[2])
            if _r1 > _r2:
                _rmax = _r1
            else:
                _rmax = _r2
            
            for _ri in range(_rmax):
                _lxtot +=1
                _rec = _recpre
                if _rec: _rec += ';'
                else: _rec = ''
                _rec += str(ret[0])+";"+str(_lxtot)
                
                # STDOUT
                if len(ret[1]) > _ri: 
                    if _rec: _rec += ';'
                    else: _rec = ''
                    _rec += str(_lx1)+";"+str(ret[1][_ri])
                    _lx1 += 1
                else:
                    if _rec: _rec += ';;'
                    else: _rec = ';'
                    
                # STDERR
                if len(ret[2]) > _ri:    
                    if _rec: _rec += ';'
                    else: _rec = ''
                    _rec += str(_lx2)+";"+str(ret[2][_ri])
                    _lx2 += 1
                else:
                    if _rec: _rec += ';;'
                    else: _rec = ';'
                
                _output(str(_rec)+'\n')

        if _t == _O_STR:
            return self._oc 
        return True

    def get_proceed(self,s): 
        """Verifies valid proceed type."""
        if s in ('print','trace', 'doit'):
            return s
        else:
            raise Exception('STATE:ERROR:proceed not supported:'+str(s))
    
    def _mode_batch(self,callstr,**kargs):
        """Internal call reference for processing a batch mode call. 
        
        The IO by stdout/stdin/stderr are redirected. Supports 
        shell-style parameters only.

        Args:
            callstr: a prepared shell-style call.
            
            **kargs:

                env: Current environment.
                    
        Returns:
            When successful returns 'True', else returns either 'False', or
            raises an exception.

        Raises:
            for parameter 'errasexcept': SystemCallsExceptionSubprocessError
            passed through exceptions:
        
        For further information refer to 'console' option of constructor/setkargs.

        """
        if self.emptyiserr and ( not callstr or callstr == ''):
            return [2,'', 'ERROR:MissingCallstr']

        ret=[1,]
        _env = kargs.get('env')
        if not _env:
            self.p=subprocess.Popen(
                    callstr,
                    shell=True,
                    bufsize=self.bufsize,
                    stdin=subprocess.PIPE,
                    stdout=subprocess.PIPE,
                    stderr=subprocess.PIPE,
                    close_fds=True
                    )
        else:
            self.p=subprocess.Popen(
                    callstr,
                    shell=True,
                    bufsize=self.bufsize,
                    stdin=subprocess.PIPE,
                    stdout=subprocess.PIPE,
                    stderr=subprocess.PIPE,
                    close_fds=True,
                    env=_env
                    )

        # fetch the results from subprocess
        self.stdin=self.p.stdin
        self.stdout=self.p.stdout
        self.stderr=self.p.stderr
        self.res=self.p.communicate()
        self.p.poll()

        _errcond = False
        if self.useexit and self.usestderr:
            if self.p.returncode or self.res[1]:
                _errcond = True
        elif self.useexit and self.p.returncode:
            _errcond = True
        elif self.usestderr and self.res[1]:
            _errcond = True

        if _errcond and self.errasexcept: # transforms errors from subprocesses into Exceptions
            # flush cache...
            if self.res[0]:
                sys.stdout.write(self.res[0])
            if self.res[1]:
                sys.stderr.write(self.res[1])
            # .. raise exception 
            #raise SystemCallsExceptionSubprocessError(self.p.returncode,**{'exitmsg':"FROM:"+str(callstr) +"\n"+ str(self.res[1]),'exitval':self.p.returncode,})
            raise SystemCallsExceptionSubprocessError(self.p.returncode,**{'exitmsg':"FROM:"+str(callstr) +"\n",})
        
        elif _errcond and self.passerr: # passes errors from subprocesses simply through as error exit
            # flush cache...
            if self.res[0]: # stdout
                sys.stdout.write(self.res[0])
            if self.res[1]: # stderr
                sys.stderr.write(str(self.res[1]))
            # ..now exit
            sys.exit(self.p.returncode)
        
        elif _errcond and not self.p.returncode and self.usestderr: # treats any presence of stderr as error
            ret[0] = 1
            ret.extend(self.res)
            return ret

        else: # wraps result of subprocess call into a tuple 
            ret[0]=self.p.returncode
            ret.extend(self.res)
            return ret

    def _mode_dialogue(self,callstr,**kargs):
        """Internal call reference for processing an interactive dialogue call.
        
        Supports shell-style parameters only.

        For further information refer to 'console' option of constructor/setkargs.

        """
        
        #FIXME: has to be tested
        if self.emptyiserr and ( not callstr or callstr == ''):
            return [2,'', 'ERROR:MissingCallstr']
        
        ret=[0,]
        try:
            self.p=subprocess.check_call(
                callstr,
                shell=True,
                bufsize=self.bufsize
                )
        except subprocess.CalledProcessError as e:
            try:
                ret[0]=self.p.returncode
                ret.append(self.p.output)
            except:
                ret[0]=99
                ret.append(str(e))
                pass
        return ret

    def setkargs(self,**kargs):
        """Sets provided parameters for the subprocess call context.
        
        Applicable for the initial call of self.__init__(), 
        and later modification. Called for each start of
        a subprocess in order to update optional the specific 
        call context modification.
        
        Args:
            **kargs: Parameter specific for the operation,

                bufsize: The size of the output buffer for the 
                    called subprocess. Refer to **subprocess.Popen**.
                    Default value is -1.

                console ('cli','dialogue')
                    ffs ('batch','ui','gtk', 'qt')

                    cli: Works in batch mode, particularly the
                        stdin, stdout, and stderr streams are
                        caught into a string buffer by the 
                        calling process via a pipe. The content
                        is passed after termination of the called 
                        sub-process.

                    dialogue: Works without buffered io streams.
                        Thus allows for interaction, but not
                        post-processing.

                    verbose: Verbose.

                debug: Sets debug for rule data flow.

                emptyiserr: Treats passed empty call strings as error.
                    The applied 'subprocess.Popen()' treats them as
                    success, which may cover errors in generated
                    call strings, particularly in loops.

                    default := False
 
                errasexcept: Passes errors as exceptions, transforms the resuls from
                    subprocesses into Exceptions data. Exits the process.

                    default := False

                out: Output for display. Supported types are:
                
                    csv: CSV seperated by ';', with sparse records
                    
                    pass: Pass through STDOUT and STDERR from 
                        subprocess

                    repr: Python 'repr()'
        
                    str: Python 'str()'

                    xml: XML

                passerr: Passes errors from subprocesses transparently 
                    through by stdout, stderr, and exit code. Exits 
                    the process.

                    default := False
 
                proceed ('print','trace', 'doit')
                    print - trace only
                    
                    trace - execute and trace
                    
                    doit  - execute    

                raw: Pass through STDOUT and STDERR.

                rules: Sets the rules object.

                useexit: Use exit code for error detection of
                    subprocess.
                    
                    default := True 

                usestderr: Use 'sys.stderr' output for error 
                    detection of subprocess. When set to 'True',
                    the presence of a string is treated as error
                    condition.
                    
                    default := False

                verbose: Sets verbose for rule data flow.

        Returns:
            When successful returns 'True', else returns either 'False', or
            raises an exception.
            Success is the complete addition only, thus one failure returns
            False.

        Raises:
            passed through exceptions:
            
        """
        for k,v in kargs.iteritems():
            if k=='bufsize':
                self.bufsize=int(v)
            elif k=='console':
                self.console=v
                if v == 'cli':
                    self.out = v
                if v == 'dialogue':
                    self.myexe=self._mode_dialogue
                elif v=='batch':
                    raise Exception('STATE:ERROR:console not supported:'+str(v))
                elif v in ('ui','gtk', 'qt'):
                    pass
                else:
                    self.myexe=self._mode_batch
            elif k=='debug':
                self.debug=v
            elif k=='emptyiserr':
                self.emptyiserr=v
            elif k=='env': # to be evaluated by the caller case by case only
                pass
            elif k=='errasexcept':
                self.errasexcept=v
            elif k=='out':
                self.out = v
                if v in ('csv', 'pass', 'repr', 'str', 'xml', ):
                    self.out = v
                else:
                    raise SystemCallsException("Unknown output type:"+str(self.out))
            elif k=='passerr':
                self.passerr=v
            elif k=='proceed':
                self.proceed=self.get_proceed(v)
            elif k=='raw':
                self.raw=v
            elif k=='rules':
                self.rules=v
            elif k=='useexit':
                self.useexit=v
            elif k=='usestderr':
                self.usestderr=v
            elif k=='verbose':
                self.verbose=v

#             else:
#                 raise Exception('STATE:ERROR:parameter not supported:'+str(k))
        return True

    def splitLines(self,oldres):
        """Converts the raw string fields including '\n' of a return value into line arrays.
        
        Args:
            oldres: The raw result of a previous call.
            
        Returns:
            Result of call, the format is:
            
                ret[0]::=exit value
                
                ret[1]::=list of lines from former stdout output, each as 
                    a partial non-processed string.
                
                ret[2]::=list of lines from former stderr output, each as
                    a partial non-processed string.

        Raises:
            passed through exceptions:
            
        """
        res = [oldres[0],[],[]]
        
        if len(oldres) > 1:
            res[1] = oldres[1].split('\n')
        if res[1] and res[1][-1] == '':
            res[1].pop()
        if res[1] and res[1][0] == '':
            res[1].pop(0)

        if len(oldres) > 2:
            res[2] = oldres[2].split('\n')
        if res[2] and res[2][-1] == '':
            res[2].pop()
        if res[2] and res[2][0] == '':
            res[2].pop(0)
        return res

    def __str__(self):
        """Prints preset call parameters.
        """
        ret = ""
        ret += "\nbufsize      = "+str(self.bufsize)
        ret += "\nconsole      = "+str(self.console)
        ret += "\nemptyiserr   = "+str(self.emptyiserr)
        ret += "\nerrasexcept  = "+str(self.errasexcept)
        ret += "\nmyexe        = "+str(self.myexe)
        ret += "\npasserr      = "+str(self.passerr)
        ret += "\nproceed      = "+str(self.proceed)
        ret += "\nraw          = "+str(self.raw)
        ret += "\nuseexit      = "+str(self.useexit)
        ret += "\nusestderr    = "+str(self.usestderr)
        return ret

    def __repr__(self):
        """Prints the current representation of call parameters for subprocesses.
        """
        ret = "{"
        ret += "'bufsize': "+str(self.bufsize)
        ret += ", 'console': "+str(self.console)
        ret += ", 'emptyiserr': "+str(self.emptyiserr)
        ret += ", 'errasexcept': "+str(self.errasexcept)
        ret += ", 'myexe': "+str(self.myexe.__name__)
        ret += ", 'passerr': "+str(self.passerr)
        ret += ", 'proceed': "+str(self.proceed)
        ret += ", 'raw': "+str(self.raw)
        ret += ", 'useexit': "+str(self.useexit)
        ret += ", 'usestderr': "+str(self.usestderr)
        ret += "}"
        return ret



# -*- coding: utf-8 -*-
"""The module 'epyunit.spUnittest' provides classes derived from 'unittest' for tests of arbitrary subprocesses.

"""
from __future__ import absolute_import

__author__ = 'Arno-Can Uestuensoez'
__license__ = "Artistic-License-2.0 + Forced-Fairplay-Constraints"
__copyright__ = "Copyright (C) 2010-2016 Arno-Can Uestuensoez @Ingenieurbuero Arno-Can Uestuensoez"
__version__ = '0.1.12'
__uuid__='9de52399-7752-4633-9fdc-66c87a9200b8'

__docformat__ = "restructuredtext en"

import sys,re
from types import NoneType

from unittest import TestCase
#import unittest

version = '{0}.{1}'.format(*sys.version_info[:2])
if version < '2.7': # pragma: no cover
    raise Exception("Requires Python-2.7.* or higher")


#from epyunit.SystemCalls import SystemCalls
from epyunit.SubprocUnit import SubprocessUnit,SProcUnitRules

class TestExecutableException(Exception):
    """Application error for called executable.
    """
    pass


#class TestExecutable(unittest.TestCase):
class TestExecutable(TestCase):
    """Extends TestCase for subprocesses.
    """

    def __init__(self,*args, **kargs):
        """Initializes the test case, passes parameters to 'unittest.TestCase'. 

        Args:
            **kargs: Parameter specific for the operation,
                passed through to **setkargs**.

        Returns:
            When successful returns 'True'.

        Raises:
            passed through exceptions:

        """
        super(TestExecutable,self).__init__(*args,**kargs)

        self.spunit = None
        """Subprocess unit assigned to this test case."""
        if not kargs.get('spunit'): # else shift setting into setkargs()
            self.spunit = SubprocessUnit()

        self.spcache = [0,None,None]
        """Subprocess output caches.
        """
    
        self.setkargs(**kargs) # sets parameters
        pass

    def setkargs(self,**kargs):
        """Sets provided parameters for the subprocess call context.
        
        Applicable for the initial call of self.__init__(), 
        and later modification. Called for each start of
        a subprocess in order to update optional the specific 
        call context modification.

        Args:
            **kargs: Parameters specific for the operations.

                noparent: Suppress call of parent class. 

                rules: Sets the rules object to be used.

                spunit: Sets the subprocess unit.

        Returns:
            When successful returns 'True', else returns either 'False', or
            raises an exception.

        Raises:
            passed through exceptions:
            
        """
        self.noparent = kargs.get('noparent',False)
        for k,v in kargs.items():
            if k == 'rules':
                self.spunit.setruleset(v)
            elif k == 'spunit':
                self.spunit = v


    def callSubprocess(self,callstr,**kargs):
        """Calls a subprocess and fetches the result data.

        Args:
            callstr: Complete call string for subprocess.
            
            **kargs: Pass-through parameters for 'SystemCalls.callit()'.

        Returns:
            When successful returns the tupel:

                returnvalue := (exit-value, stdout-value, stderr-value)

        Raises:
            passed through exceptions:
        
        """
        return self.spunit.callit(callstr,**kargs)

    def callSubprocessCached(self,callstr,**kargs):
        """Calls a subprocess and fetches the result data into cache for further processing.

        Args:
            callstr: Complete call string for subprocess.
            
            **kargs: Pass-through parameters for 'SystemCalls.callit()'.

        Returns:
            When successful returns the tupel:

                returnvalue := (exit-value, stdout-value, stderr-value)

        Raises:
            passed through exceptions:
        
        """
        self.spcache = self.spunit.callit(callstr,**kargs)
        return self.spcache

    def assertEqual(self,exp,cur=None):
        """Asserts on the exit value of the called process.
        """
        if cur==NoneType:
            cur = self.spcache
        super(TestExecutable,self).assertEqual(exp, cur)
        pass
    
    def assertExit(self,exp,cur=None):
        """Asserts on the exit value of the called process.
        """
        if cur==NoneType:
            cur = self.spcache[0]
        assert exp[0] == cur[0]
        pass
    
    def assertStdout(self,exp,cur=None):
        """Asserts a list of provided regexpr on the STDOUT of the called process.
        """
        if cur==NoneType:
            cur = self.spcache[1]
        super(TestExecutable,self).assertEqual(exp[1], cur[1])
        pass

    def assertStderr(self,exp,cur=None):
        """Asserts a list of provided regexpr on the STDERR of the called process.
        """
        if cur==NoneType:
            cur = self.spcache[2]
        super(TestExecutable,self).assertEqual(exp[2], cur[2])
        pass

    def __str__(self):
        """Prints the current test state.
        """
        pass



#!/usr/bin/env python
from __future__ import absolute_import
# from __future__ import print_function

__author__ = 'Arno-Can Uestuensoez'
__license__ = "Artistic-License-2.0 + Forced-Fairplay-Constraints"
__copyright__ = "Copyright (C) 2016 Arno-Can Uestuensoez @Ingenieurbuero Arno-Can Uestuensoez"
__version__ = '0.1.12'
__uuid__ = '9de52399-7752-4633-9fdc-66c87a9200b8'

__docformat__ = "restructuredtext en"

import sys

_rdbg = None
_rdbg_default = "localhost:5678"  # the defaults as defined by PyDev

if "--rdbg" in sys.argv:
    _ai = sys.argv.index("--rdbg")
    _a = sys.argv.pop(_ai)  # --rdbg
    if _ai < len(sys.argv) and sys.argv[_ai][:1] != '-':
        _rdbg = sys.argv.pop(_ai)  # value
    else:
        _rdbg = _rdbg_default

    import epyunit.PyDevERDbg
    epyunit.PyDevERDbg.PYDEVD.startDebug()  # start debugging here...
    #
    # remote breakpoints could be set from here on...
    #

for ax in sys.argv[1:]:
    ax = ax.upper()
    if ax in ("-h", "-help", "--help",):

        print """
            Provided test cases: ( OK, NOK, PRIO, EXITOK, EXITNOK, EXIT7, EXIT8, EXIT9OK3NOK2, STDERRONLY, DEFAULT )
            
            # A: succeed: OK
              EXIT:
                0
              STDOUT:
                fromA
                arbitrary output
                arbitrary signalling OK string
                arbitrary output
              STDERR:
                -
            
            # B: fail: NOK
               EXIT:
                 0
               STDOUT:
                 fromB
                 arbitrary output
                 arbitrary output
               STDERR:
                 arbitrary signalling ERROR string
            
            # C: redundancy resolved by user defined priority: PRIO
               EXIT:
                 0
               STDOUT:
                 fromC
                 arbitrary output
                 arbitrary signalling OK string
                 arbitrary output
               STDERR:
                 arbitrary signalling ERROR string
            
            
            # D: exit value: EXITOK
               EXIT:
                 0
               STDOUT:
                 fromD
                 arbitrary output
                 arbitrary signalling OK string
                 arbitrary output
               STDERR:
                 -
            
            # E: exit value: EXITNOK
               EXIT:
                 1
               STDOUT:
                 fromE
                 arbitrary output
                 arbitrary signalling OK string
                 arbitrary output
               STDERR:
                 -
            
            # F: exit value: EXIT7
               EXIT:
                 7
               STDOUT:
                 fromF
                 arbitrary output
                 arbitrary signalling NOK string
                 arbitrary output
               STDERR:
                 -
            
            # G: exit value: EXIT8
               EXIT:
                 8
               STDOUT:
                 fromG
                 arbitrary output
                 arbitrary signalling NOK string
                 arbitrary output
               STDERR:
                 arbitrary err output
                 arbitrary err signalling NOK string
                 arbitrary err output
            
            # H: exit value: EXIT9OK3NOK2
               EXIT:
                 9
               STDOUT:
                 fromG
                 OK
                 OK
                 OK
               STDERR:
                 NOK
                 NOK
            
            # I: exit value: STDERRONLY
               EXIT:
                0
               STDOUT:
                -
               STDERR:
                 NOK
                 NOK
            
            # DEFAULT: define: here succeed '--default-ok': DEFAULT
               EXIT:
                 123
               STDOUT:
                 arbitrary output
               STDERR:
                 -
            	
        """
        sys.exit(0)

    elif ax in ("OK",): 
        # A: succeed
        print "fromA"
        print "arbitrary output"
        print "arbitrary signalling OK string"
        print "arbitrary output"
        sys.exit(0)

    elif ax in ("NOK",):
        # B: fail
        print "fromB"
        print "arbitrary output"
        print >> sys.stderr, "arbitrary signalling ERROR string"
        print "arbitrary output"
        sys.exit(0)

    elif ax in ("PRIO",):
        # C: redundancy resolved by user defined priority
        print "fromC"
        print "arbitrary output"
        print "arbitrary signalling OK string"
        print "arbitrary output"
        print >> sys.stderr, "arbitrary signalling ERROR string"
        sys.exit(0)

    elif ax in ("EXITOK",):
        # D: exit value
        print "fromD"
        print "arbitrary output"
        print "arbitrary signalling OK string"
        print "arbitrary output"
        sys.exit(0)

    elif ax in ("EXITNOK",):
        # E: exit value
        print "fromE"
        print "arbitrary output"
        print "arbitrary signalling OK string"
        print "arbitrary output"
        sys.exit(1)

    elif ax in ("EXIT7",):
        # F: exit value
        print "fromF"
        print "arbitrary output"
        print "arbitrary signalling NOK string"
        print "arbitrary output"
        sys.exit(7)

    elif ax in ("EXIT8",):  # G: exit value
        print "fromG"
        print "arbitrary output"
        print "arbitrary signalling NOK string"
        print "arbitrary output"
        print >> sys.stderr, "arbitrary err output"
        print >> sys.stderr, "arbitrary err signalling NOK string"
        print >> sys.stderr, "arbitrary err output"
        sys.exit(8)

    elif ax in ("EXIT9OK3NOK2",):
        # H: exit value
        print "fromH"
        print "OK"
        print "OK"
        print "OK"
        print >> sys.stderr, "NOK"
        print >> sys.stderr, "NOK"
        sys.exit(9)

    elif ax in ("STDERRONLY",):
        # I: exit value
        print >> sys.stderr, "fromI"
        print >> sys.stderr, "NOK"
        print >> sys.stderr, "NOK"
        sys.exit(0)

    else:
        # DEFAULT: define: here succeed '--default-ok'
        print >>sys.stderr,"ERROR:Unknown parameter:"+str(ax)
        sys.exit(123)

# DEFAULT: define: here succeed '--default-ok'
print "arbitrary output"
sys.exit(123)



# -*- coding: utf-8 -*-
"""The 'FileSysObjectsMin' module provides basic file inheritance by directory hierarchies.  

This modules manages the file system based structure semantics for 
the automation of drop-in configurations. Therefore functions and classes
are provided for the modification of the search path and the location of
files and directories. Two examples are:

* The search for files and/or relative paths from a given 
    directory on upwards.
* The extension of search paths with any sub-directory portion 
    from a given directory on upwards.  

These two features already provide for basic inheritance 
features for files in directory hierarchies.

In addition this module supports for the location of files and 
file positions by means of the package 'inspect'.

**REMARK**: For the full set refer to the package:
PyFileSysObjects / https://pypi.python.org/pypi/PyFileSysObjects

"""
from __future__ import absolute_import

__author__ = 'Arno-Can Uestuensoez'
__license__ = "Artistic-License-2.0 + Forced-Fairplay-Constraints"
__copyright__ = "Copyright (C) 2010-2016 Arno-Can Uestuensoez @Ingenieurbuero Arno-Can Uestuensoez"
__version__ = '0.0.1'
__uuid__='9de52399-7752-4633-9fdc-66c87a9200b8'

__docformat__ = "restructuredtext en"

import os,sys
version = '{0}.{1}'.format(*sys.version_info[:2])
if version < '2.7': # pragma: no cover
    raise Exception("Requires Python-2.7.* or higher")

import inspect 


class FileSysObjectsMinException(Exception):
    pass

class FileSysObjectsMin:
    pass

def setUpperTreeSearchPath(start=None,top=None,plist=None):
    """Extends the search by sys.path for each subdir from 'start' on upward to 'top'.
    
    Prepends a set of search paths into sys.path. The set of search 
    paths contains of each directory beginning with provided start 
    position. This enables a "polymorphic inheritance view" onto the 
    file system structure.
    
    The typical application is the grouping of imported modules/classes
    for test cases by file-inheritance from upper tree. This provides 
    the hierarchical configuration and specialization of test cases 
    with pre-defined default components by multiple levels of 
    configuration files. 

    Args:
        start: Start directory.

            default := caller-file-position.
        
        top: End directory.
        
            default := start

        plist: List to for the storage.
        
            default := sys.path

    Returns:
        When successful returns 'True', else returns either 'False', or
        raises an exception.

    Raises:
        passed through exceptions:
    """
    if plist == None:
        plist = sys.path

    # 0. prep start dir
    if start == '':
        raise FileSysObjectsMinException("Empty top:''")
    elif start == None:
        start = getSourceFilePathName(2) # caller file
    s = os.path.abspath(start)
    if os.path.isfile(start):
        s = os.path.dirname(s) # we need dir
    if not os.path.exists(s):
        raise FileSysObjectsMinException("Missing start:"+str(s)) 

    # 1. prep top dir
    if top == '':
        raise FileSysObjectsMinException("Empty top:''")
    elif top == None:
        plist.insert(0,s)
        return True
    else:
        a = s.split(top)
        
        if len(a) == 1: # top is not in start
            raise FileSysObjectsMinException("Missing top:"+str(top))

        elif a == ['','']: # handles top==start
            plist.insert(0,top)
            return True
        
        elif len(a)>2: # multiple occurances, for not not supported
            raise FileSysObjectsMinException("Ambigious top:"+str(top)) # top more than once in start
        
        else: # exactly 2 - one only
            
            if a[0] == '': # top was prefix
                a = a[1]
                curp = top
                plist.insert(0,curp)
                a = a.split(os.sep)
            
                for p in a:
                    curp = os.path.join(curp,p)
                    if p != '':
                        curp += os.sep
                    plist.insert(0,curp)

            else: # actual top
                curp = os.path.normpath(a[0]+os.sep+top)
                plist.insert(0,curp)

                a = a[1].split(os.sep)
                            
                for p in a:
                    if not p:
                        continue
                    curp = os.path.join(curp,p)
                    if p != '':
                        curp += os.sep
                    plist.insert(0,curp)

    return True

def findRelPathInUpperTree(object,plist=None):
    """Searches the upper tree path list for matching objects in side-branches.
    
    Args:
        object: A relative path to a directory or file.
                Some examples are 'myscript.sh', 'epyunit/myscript.sh', or
                'bin/epyunit'.

        plist: List with path entries to search each, first match wins.
        
            default := sys.path

    Returns:
        When successful returns the absolute pathname, else 'None'.

    Raises:
        passed through exceptions:
    """
    if plist == None:
        plist = sys.path
    for p in plist:
        if os.path.exists(p+os.sep+object):
            return os.path.normpath(os.path.abspath(p+os.sep+object))

    return None

def getSourceFilePathName(spos=1):
    """Returns the pathname of caller source file.

    Args:
        spos: Start position in the stack.

    Returns:
        Returns the filepathname.

    Raises:
        passed through exceptions:
    """
    return inspect.stack()[spos][1]

def getSourceFuncName(spos=1):
    """Returns the name of caller function.

    Args:
        spos: Start position in the stack.

    Returns:
        Returns the filepathname.

    Raises:
        passed through exceptions:
    """
    return inspect.stack()[spos][3]

def getSourceLinenumber(spos=1):
    """Returns the line number of caller.

    Args:
        spos: Start position in the stack.

    Returns:
        Returns the filepathname.

    Raises:
        passed through exceptions:
    """
    return inspect.stack()[spos][2]

def getCallerName(spos=1):
    """Returns the name of the caller.

    Args:
        spos: Start position in the stack.

    Returns:
        Returns the filepathname.

    Raises:
        passed through exceptions:
    """
    return inspect.stack()[spos][0].f_globals['__name__']

def getCallerNameOID(spos=1):
    """Returns the name of the package containing the caller.

    Args:
        spos: Start position in the stack.

    Returns:
        Returns the package name.

    Raises:
        passed through exceptions:
    """
    
    #def caller_name(skip=2):
    """Get a name of a caller in the format module.class.method
    `skip` specifies how many levels of stack to skip while getting caller
    name. skip=1 means "who calls me", skip=2 "who calls my caller" etc.
    An empty string is returned if skipped levels exceed stack height
    """
    _spos = spos
    _sf = inspect.stack()
    if len(_sf) < _spos:
        return None
    _pf = _sf[_spos][0]
    _coid = inspect.getmodule(_pf)
    if _coid:
        _coid = _coid.__name__
    if 'self' in _pf.f_locals:
        _coid += "." + str(_pf.f_locals['self'].__class__.__name__)
    if _pf.f_code.co_name != '<module>':
        _coid += "." + str(_pf.f_code.co_name)
    del _pf
    return _coid

def getCallerPackageName(spos=1):
    """Returns the name of the package containing the caller.

    Args:
        spos: Start position in the stack.

    Returns:
        Returns the package name when defined, else None.

    Raises:
        passed through exceptions:
    """
    _sf = inspect.stack()
    if len(_sf) < spos:
        return None
    module = inspect.getmodule(_sf[spos][0])
    if module:
        return module.__package__
    
def getCallerPackagePathName(spos=1):
    """Returns the pathname to the package directory of the caller.

    Args:
        spos: Start position in the stack.

    Returns:
        Returns the path name to the package.

    Raises:
        passed through exceptions:
    """
    _sf = inspect.stack()
    if len(_sf) < spos:
        return None
    module = inspect.getmodule(_sf[spos][0])
    if module:
        if module.__package__:
            #FIXME: zip-files
            return getCallerModulePathName(spos+1)

def getCallerPackagePythonPath(spos=1):
    """Returns the pathname to the package directory of the caller.

    Intentionally the same as 'getCallerPackagePathName'.
 
    Args:
        spos: Start position in the stack.

    Returns:
        Returns the path name to the package.

    Raises:
        passed through exceptions:
    """
    return os.path.dirname(getCallerPackagePathName(spos+1))

def getCallerPackageFilePathName(spos=1):
    """Returns the name of the package containing the caller.

    Args:
        spos: Start position in the stack.

    Returns:
        Returns the package name.

    Raises:
        passed through exceptions:
    """
    return os.path.dirname(getCallerModuleFilePathName(spos+1))

def getCallerModuleFilePathName(spos=1):
    """Returns the filepathname of the module.

    Args:
        spos: Start position in the stack.

    Returns:
        Returns the filepathname of the caller module.

    Raises:
        passed through exceptions:
    """
    
    _sf = inspect.stack()
    if len(_sf) < spos:
        return None
    module = inspect.getmodule(_sf[spos][0])
    if module:
        return module.__file__

def getCallerModuleName(spos=1):
    """Returns the name of the caller module.

    Args:
        spos: Module name.

    Returns:
        Returns the name of caller module.
        The dotted object path is relative to 
        the actual used sys.path item.  

    Raises:
        passed through exceptions:
    """
    _sf = inspect.stack()
    if len(_sf) < spos:
        return None
    module = inspect.getmodule(_sf[spos][0])
    if module:
        return module.__name__

def getCallerModulePythonPath(spos=1):
    """Returns the prefix item from sys.path used for the caller module.

    Args:
        spos: Module name.

    Returns:
        Returns the name of caller module.

    Raises:
        passed through exceptions:
    """
    _mn = getCallerModuleName(spos+1).split('.')[:-1]
    _mn = os.sep.join(_mn)
    _r = getCallerModulePathName(spos+1).split(_mn)[0]
    if _r[-1] != os.sep: # was rel for split
        _r += os.sep
    return _r    

def getCallerModulePathName(spos=1):
    """Returns the pathname of the module.

    Args:
        m: Module name.

    Returns:
        Returns the filepathname of module.

    Raises:
        passed through exceptions:
    """
    return os.path.dirname(getCallerModuleFilePathName(spos+1))+os.sep

def getPythonPathPrefixMatchFromSysPath(pname,plist=None):
    """Gets the first matching prefix from sys.path.
    
    Foreseen to be used for canonical base reference in unit tests.
    This enables in particular for casual tests where absolute pathnames
    are required.

    Args:
        pname: Pathname.

    Returns:
        Returns the first matching path prefix from sys.path.

    Raises:
        passed through exceptions:
    """
    if not plist:
        plist = sys.path
    for sp in plist:
        if pname.startswith(sp):
            if sp and sp[-1] == os.sep:
                return sp
            return sp+os.sep

def getPythonPathModuleRel(fpname,plist=None):
    """Returns the relative path name of the filepathname.

    Args:
        fpname: The filepathname.

    Returns:
        Returns the path prefix for fpname.

    Raises:
        passed through exceptions:
    """
    if not plist:
        plist = sys.path
    for _sp in plist:
        if fpname.startswith(_sp):
            _r = fpname.replace(_sp,"")
            if _r and _r[0] == os.sep:
                return _r[1:]
            return _r

def getPythonPathForPackage(pname,plist=None):
    """Returns the item from sys.path for package.

    ATTENTION: This version relies on the common
        naming convention of pathnames.

    Args:
        pname: The name of the package.

    Returns:
        Returns the path item.

    Raises:
        passed through exceptions:
    """
    if not plist:
        plist = sys.path    
    _pn = pname.replace('.',os.sep)
    for _sp in sys.path:
        _pp = _sp.split(pname)
        if len(_pp) > 1:
            return _pp[0]



# -*- coding: utf-8 -*-
"""Selftest for the package 'epyunit'.

"""
from __future__ import absolute_import

__author__ = 'Arno-Can Uestuensoez'
__license__ = "Artistic-License-2.0 + Forced-Fairplay-Constraints"
__copyright__ = "Copyright (C) 2010-2016 Arno-Can Uestuensoez @Ingenieurbuero Arno-Can Uestuensoez"
__version__ = '0.1.12'
__uuid__='9de52399-7752-4633-9fdc-66c87a9200b8'

__docformat__ = "restructuredtext en"


import sys,os

from epyunit.SubprocUnit import SubprocessUnit,SProcUnitRules

_appname = "selftest" #: application name

def _subcall_myscript(slang,**kargs):
    """Calls the operational checks provided by 'myscript' for language variant.  

    Args:
        slang: Programming language of called script, current are: bash, python
        

        **kargs:

            out:
            
            verbose:

            debug:

    Returns:
        None.

    Raises:
        In case of assertion errors.

    """
    _out = kargs.get('out',None)
    _verbose = kargs.get('verbose',0)
    _debug = kargs.get('debug',0)
    
    if _verbose>0:
        kargs['verbose'] = _verbose 
    if kargs.get('debug') > 0:
        kargs['debug'] = _debug 

    # create a complete subprocess unit test controller with configuration data
    sx = SubprocessUnit()
    import epyunit #for path to mymodule.sh

    if slang == 'bash':
        _sx = "myscript.sh"
        cx = epyunit.__path__[0]+os.sep+_sx
    elif slang == 'python':
        _sx = "myscript.py"
        cx = epyunit.__path__[0]+os.sep+_sx
    else:
        raise Exception("Unknown language type:"+str(slang))
    
    # add rules to be applied ont the subprocesses
    _myRules = SProcUnitRules()


    #
    # perform the tests for defined filter
    #
    
    #
    #*******
    #
    _myParams = {'exitval':123,'stdoutok':["arbitrary output"],'stderrok':[],}
    _myrx = {'reset':True, 'exitign': False, 'exitval': 123, 'exittype':'VAL',}
    _myRules.setrules(**_myrx)
    _myRules.setkargs(**_myParams)
    if _verbose >1:
        print _myRules 
    sx.setkargs(**{'rules':_myRules,})
    ret = sx.callit(cx)
    if ret[0] == 126:
        print >>sys.stderr ,"check exec permissions of 'myscript.sh'"
    if _out:
        if _verbose:
            print "\n#*** epyunit/"+str(_sx)+" DEFAULT ***" 
        if _verbose>1:
            sx.displayit(ret)                   

    retX = [123,["arbitrary output"],[]]
    try:
        assert ret ==retX
    except:
        print >>sys.stderr, ""
        print >>sys.stderr, "retX="+str(retX)
        print >>sys.stderr, "ret= "+str(ret)
    assert sx.apply(ret)


    #
    #*******
    #
    _myParams = {'exitval':0,'stdoutok':["fromA", "arbitrary output","arbitrary signalling OK string","arbitrary output"],'stderrok':[],}
    _myrx = {'reset':True, 'exitign': False, 'exitval': 0, 'exittype':'OK', 'stdoutok_val':["arbitrary output","arbitrary signalling OK string","arbitrary output"]}
    _myRules.setrules(**_myrx)
    _myRules.setkargs(**_myParams)
    if _verbose>1:
        print _myRules 
    sx.setkargs(**{'rules':_myRules,})
    ret = sx.callit(cx+' OK')
    if _out:
        if _verbose:
            print "\n#*** epyunit/"+str(_sx)+" OK ***" 
        if _verbose>1:
            sx.displayit(ret)                   
    retX = [0,["fromA", "arbitrary output","arbitrary signalling OK string","arbitrary output"],[]]
    try:
        assert ret ==retX
    except:
        print >>sys.stderr, ""
        print >>sys.stderr, "retX="+str(retX)
        print >>sys.stderr, "ret= "+str(ret)
    assert sx.apply(ret)



    #
    #*******
    #
    _myParams = {'exitval':0,'stdoutok':["fromC", "arbitrary output","arbitrary signalling OK string","arbitrary output"],'stderrok':[],}
    _myrx = {'reset':True, 'exitign': False, 'exittype':'OK','stdoutok':["arbitrary output","arbitrary signalling OK string","arbitrary output"]}
    _myRules.setrules(**_myrx)
    _myRules.setkargs(**_myParams)
    if _verbose>1:
        print _myRules 
    sx.setkargs(**{'rules':_myRules,})
    ret = sx.callit(cx+' PRIO')
    if _out:
        if _verbose:
            print "\n#*** epyunit/"+str(_sx)+" PRIO ***" 
        if _verbose>1:
            sx.displayit(ret)                   
    retX = [0,["fromC", "arbitrary output","arbitrary signalling OK string","arbitrary output"],["arbitrary signalling ERROR string"]]
    try:
        assert ret ==retX
    except:
        print >>sys.stderr, ""
        print >>sys.stderr, "retX="+str(retX)
        print >>sys.stderr, "ret= "+str(ret)
    assert sx.apply(ret)



    #
    #*******
    #
    _myParams = {'exitval':0,'stdoutok':["fromD", "arbitrary output","arbitrary signalling OK string","arbitrary output"],'stderrok':[],}
    _myrx = {'reset':True, 'exitign': False, 'exittype':'OK','stdoutok':["arbitrary output","arbitrary signalling OK string","arbitrary output"]}
    _myRules.setrules(**_myrx)
    _myRules.setkargs(**_myParams)
    if _verbose>1:
        print _myRules 
    sx.setkargs(**{'rules':_myRules,})
    ret = sx.callit(cx+' EXITOK')
    if _out:
        if _verbose:
            print "\n#*** epyunit/"+str(_sx)+" EXITOK ***" 
        if _verbose>1:
            sx.displayit(ret)                   
    retX = [0,["fromD", "arbitrary output","arbitrary signalling OK string","arbitrary output"],[]]
    try:
        assert ret ==retX
    except:
        print >>sys.stderr, ""
        print >>sys.stderr, "retX="+str(retX)
        print >>sys.stderr, "ret= "+str(ret)
    assert sx.apply(ret)



    #
    #*******
    #
    _myParams = {'exitval':1,'stdoutok':["fromD", "arbitrary output","arbitrary signalling OK string","arbitrary output"],'stderrok':[],}
    _myrx = {'reset':True, 'multiline':True, 'exitign': False, 'exittype':'NOK','stdoutok':["arbitrary output","arbitrary signalling OK string","arbitrary output"]}
    _myRules.setrules(**_myrx)
    _myRules.setkargs(**_myParams)
    if _verbose>1:
        print _myRules 
    sx.setkargs(**{'rules':_myRules,})
    ret = sx.callit(cx+' EXITNOK')
    if _out:
        if _verbose:
            print "\n#*** epyunit/"+str(_sx)+" EXITNOK ***" 
        if _verbose>1:
            sx.displayit(ret)                   
    retX = [1,["fromE", "arbitrary output","arbitrary signalling OK string","arbitrary output"],[]]
    try:
        assert ret ==retX
    except:
        print >>sys.stderr, ""
        print >>sys.stderr, "retX="+str(retX)
        print >>sys.stderr, "ret= "+str(ret)
    assert sx.apply(ret)



    #
    #*******
    #
    _myParams = {'exitval':7,'stdoutok':["fromF", "arbitrary output","arbitrary signalling NOK string","arbitrary output"],'stderrok':[],}
    _myrx = {'reset':True, 'exitign': False, 'exittype':'VAL','exitval':7,'stdoutok':["arbitrary output","arbitrary signalling OK string","arbitrary output"]}
    _myRules.setrules(**_myrx)
    _myRules.setkargs(**_myParams)
    if _verbose>1:
        print _myRules 
    sx.setkargs(**{'rules':_myRules,})
    ret = sx.callit(cx+' EXIT7')
    if _out:
        if _verbose:
            print "\n#*** epyunit/"+str(_sx)+" EXIT7 ***" 
        if _verbose>1:
            sx.displayit(ret)                   
    retX = [7,["fromF", "arbitrary output","arbitrary signalling NOK string","arbitrary output"],[]]
    try:
        assert ret ==retX
    except:
        print >>sys.stderr, ""
        print >>sys.stderr, "retX="+str(retX)
        print >>sys.stderr, "ret= "+str(ret)
    assert sx.apply(ret)



    #
    #*******
    #
    _myParams = {'exitval':8,'stdoutok':["fromG", 'arbitrary output', 'arbitrary signalling NOK string', 'arbitrary output'],'stderrok': ['arbitrary err output', 'arbitrary err signalling NOK string', 'arbitrary err output'],}
    _myrx = {'reset':True, 'exitign': False, 'exittype':'VAL','exitval':8,'stdoutok':["arbitrary output","arbitrary signalling OK string","arbitrary output"]}
    _myRules.setrules(**_myrx)
    _myRules.setkargs(**_myParams)
    if _verbose>1:
        print _myRules 
    ret = sx.callit(cx+' EXIT8')
    if _out:
        if _verbose:
            print "\n#*** epyunit/"+str(_sx)+" EXIT8 ***" 
        if _verbose>1:
            sx.displayit(ret)                   
    retX = [8,["fromG", 'arbitrary output', 'arbitrary signalling NOK string', 'arbitrary output'], ['arbitrary err output', 'arbitrary err signalling NOK string', 'arbitrary err output']]
    try:
        assert ret ==retX
    except:
        print >>sys.stderr, ""
        print >>sys.stderr, "retX="+str(retX)
        print >>sys.stderr, "ret= "+str(ret)
    assert sx.apply(ret)


    #
    #*******
    #
    _myParams = {'exitval':123,'stdoutok':["arbitrary output"],'stderrok': [],}
    _myrx = {'reset':True, 'exitign': False, 'exittype':'NOK','stdoutok':["arbitrary output","arbitrary signalling OK string","arbitrary output"]}
    _myRules.setrules(**_myrx)
    _myRules.setkargs(**_myParams)
    if _verbose>1:
        print _myRules 
    ret = sx.callit(cx)
    if _out:
        if _verbose:
            print "\n#*** epyunit/"+str(_sx)+" DEFAULT ***" 
        if _verbose>1:
            sx.displayit(ret)                   
    retX = [123,["arbitrary output"],[]]
    try:
        assert ret ==retX
    except:
        print >>sys.stderr, ""
        print >>sys.stderr, "retX="+str(retX)
        print >>sys.stderr, "ret= "+str(ret)
    assert sx.apply(ret)

    pass

def selftest(**kargs):
    """Calls some interfaces for basic operational checks.  

    Args:
        callstr: a prepared shell-style call.
        
        **kargs:

            out:
            
            verbose:

            debug:

    Returns:
        None.

    Raises:
        In case of assertion errors.

    """
    
    _subcall_myscript('bash',**kargs)    
    _subcall_myscript('python',**kargs)    
    pass



# -*- coding: utf-8 -*-
from __future__ import absolute_import

__author__ = 'Arno-Can Uestuensoez'
__license__ = "Artistic-License-2.0 + Forced-Fairplay-Constraints"
__copyright__ = "Copyright (C) 2010-2016 Arno-Can Uestuensoez @Ingenieurbuero Arno-Can Uestuensoez"
__version__ = '0.1.12'
__uuid__='9de52399-7752-4633-9fdc-66c87a9200b8'

__docformat__ = "restructuredtext en"

import sys,re
from types import NoneType

version = '{0}.{1}'.format(*sys.version_info[:2])
if version < '2.7': # pragma: no cover
    raise Exception("Requires Python-2.7.* or higher")

from epyunit.SystemCalls import SystemCalls

_parentrepr = re.compile(ur'[{](.*[^}])}')
"""
static precompiled 're' for parent '__repr__'.
"""

#
# enums for priority type 
#
_P_OK = 0 
"""
hasSuccess
"""

_P_NOK = 1
"""
hasFailure
"""

_P_WEIGHT = 3
"""
simple counter compare, bigger wins: (resultok, resultnok), else default
"""
_prioenums = {0:'_P_OK',1:'_P_NOK',3:'_P_WEIGHT', }

#
# enums for exit type 
#
_E_IGN = 4
"""
exittype: ignore
"""

_E_OK = 8
"""
exittype: OK
"""

_E_NOK = 16
"""
exittype: NOK
"""

_E_VAL = 32
"""
exittype: value
"""
_exitenums = {4:'_E_IGN', 8:'_E_OK', 16:'_E_NOK',32:'_E_VAL', }


class SProcUnitRulesException(Exception):
    """Application error of unittest rules.
    """
    pass

class SubprocessUnitException(Exception):
    """Failure of subprocess call.
    """
    pass

class SProcUnitRules(object):
    """
    The 'epyunit.SubprocUnit.SProcUnitRules' defines the expected data, and
    the minimal degree of expectation. Therefore a set of parameters 
    constitute a basic rule set, which matches the provided reference data
    on the actual response data.  
    """

    rules_default = {
        # default when ambiguous
        'default': True,
        
        # re flags
        'cflags': 0,
        'multiline': 0,
        'ignorecase': 0,
        'unicode': 0,
        'dotall': 0,
        'debug': 0,

        # priorities
        'priotype': _P_NOK,

        # required result counters
        'result': 0,
        'resultok': 0,
        'resultnok': 0,

        # exit value match
        'exitign': False,
        'exittype': _E_OK,
        'exitval': 0,

        # stderr match
        'stderrchk': False,
        'stderrnok': [],
        'stderrok': [],

        # stdout match
        'stdoutchk': False,
        'stdoutnok': [],
        'stdoutok': [],

        '_exitcond': False,
        '_exitret': 0,

    }
    """
    The default rules for OK operations
    """

    def __init__(self,**kargs):
        """Initializes the ruleset and operational parameters.
        
        For parameters refer to called **setrules** and **setkargs**.
        """
        #
        # the re pattern
        #
        self.stderrnok = []; """list of 're' patterns indicating error on STDERR"""
        self.stderrok = []; """list of 're' patterns indicating success on STDERR"""
        self.stdoutnok = []; """list of 're' patterns indicating success on STDOUT"""
        self.stdoutok = []; """list of 're' patterns indicating success on STDOUT"""

        #
        # the actual matched strings
        #
        self.stderrnok_matched = []; """list of actually matched failure strings on STDERR"""
        self.stderrok_matched = []; """list of actually matched success strings on STDERR"""
        self.stdoutnok_matched = []; """list of actually matched failure strings on STDOUT"""
        self.stdoutok_matched = []; """list of actually matched success strings on STDOUT"""

        self.reset()
        
        _args = kargs.copy()
        self.setkargs(**_args) 
        if not _args.get('default'): # for initial default
            self.setrules(**{'default':True}) 
        self.setrules(**_args) 

    def reset(self):
        """A completely empty initial in-place default set for current variables. 
        """
        # default for ambiguity
        self.default = self.rules_default['default']

        # re flags
        self.cflags = self.rules_default['cflags']
        self.multiline = self.rules_default['multiline']
        self.ignorecase = self.rules_default['ignorecase']
        self.unicode = self.rules_default['unicode']
        self.dotall = self.rules_default['dotall']
        self.debug =  self.rules_default['debug']

        # priorities
        self.priotype = self.rules_default['priotype']

        # required results
        self.result_cnt = 0
        self.resultok_cnt = 0
        self.resultnok_cnt = 0
        self.result = self.rules_default['result']
        self.resultok = self.rules_default['resultok']
        self.resultnok = self.rules_default['resultnok']

        # exit value match
        self.exitign = self.rules_default['exitign']
        self.exittype = self.rules_default['exittype']
        self.exitval = self.rules_default['exitval']

        # stderr match pattern
        self.stderrchk = self.rules_default['stderrchk']
        lx=len(self.stderrnok)
        for x in range(lx): #@UnusedVariable
            self.stderrnok.pop()
        self.stderrnok.extend(self.rules_default['stderrnok'])

        lx=len(self.stderrok)
        for x in range(lx): #@UnusedVariable
            self.stderrok.pop()
        self.stderrok.extend(self.rules_default['stderrok'])
        
        # stderr matched strings
        lx=len(self.stderrnok_matched)
        for x in range(lx): #@UnusedVariable
            self.stderrnok_matched.pop()
        self.stderrnok_cnt = 0

        lx=len(self.stderrok_matched)
        for x in range(lx): #@UnusedVariable
            self.stderrok_matched.pop()
        self.stderrok_cnt = 0

        
        # stdout match pattern
        self.stdoutchk = self.rules_default['stdoutchk']
        lx=len(self.stdoutnok)
        for x in range(lx): #@UnusedVariable
            self.stdoutnok.pop()
        self.stdoutnok.extend(self.rules_default['stdoutnok'])

        lx=len(self.stdoutok)
        for x in range(lx): #@UnusedVariable
            self.stdoutok.pop()
        self.stdoutok.extend(self.rules_default['stdoutok'])

        # stdout matched strings
        lx=len(self.stdoutnok_matched)
        for x in range(lx): #@UnusedVariable
            self.stdoutnok_matched.pop()
        self.stdoutnok_cnt = 0

        lx=len(self.stdoutok_matched)
        for x in range(lx): #@UnusedVariable
            self.stdoutok_matched.pop()
        self.stdoutok_cnt = 0

        self._exitcond = self.rules_default['_exitcond']
        self._exitret = self.rules_default['_exitret']

        pass

    def setkargs(self,**kargs):
        """Sets supported parameters.
        
        Applicable for the initial call of self.__init__(), 
        and later modification.
        
        Tolerates unknown keyword parameters, pops own.
        
        Args:
            **kargs: Parameter specific for the operation,

                debug: Sets debug for rule application.

                verbose: Sets verbose for rule application.
                
        Returns:
            When successful returns 'True', else returns either 'False',
            or raises an exception.

        Raises:
            passed through exceptions:
            
        """
        _ret = True
        for k,v in kargs.iteritems():
            if k=='debug':
                self.debug=v
            elif k=='verbose':
                self.verbose=v
        return _ret

    def setrules(self,**predef):
        """Initialize and reset parameters and previous results.

        Args:
            **kargs:

                setdefault: Sets the predefined default dictionary, 
                    see 'rules_default'.
                
                debug:

                    True: see *re.DEBUG*, when set

                    False: default:=unset

                dotall:

                    True: see *re.DOTALL*, when set

                    False: default:=unset

                exitign:
                    
                    True: Ignores exit at all.
                    
                    False: Does not ignore exit value.

                exittype:
                
                    False|'nok': Success when exit is not 0.

                    True|'ok': Success when exit is 0.
                    
                    'value': Success when exit is equal <exit-value>.

                exitval=<exit-value>: A specific exit value indicating
                    success.

                ignorecase:

                    True: see *re.IGNORECASE*, when set

                    False: default:=unset

                multiline: 

                    True: see *re.MULTILINE*, when set

                    False: default:=unset

                priotype:
                    
                    False|'nok': One failure state sets the whole case 
                        to failure. This is the default.

                    True|'ok': One success state sets the whole case 
                        to success.
                    
                    'weight': Choose larger values of counters.

                reset: A reset dictionary, see 'rules_reset'.

                result: Required sum of all matches for success, 
                    else failure.

                resultnok: Required sum of NOK matches for failure, 
                    else success.

                resultok: Required sum of OK matches for success, 
                    else failure.

                stderrnok: List of regular expressions for match on
                    STDERR, indicating failure.

                    Could contain app-cached precompiled 're'.

                stderrok: List of regular expressions for match on
                    STDERR, indicating success.

                    Could contain app-cached precompiled 're'.

                stdoutnok: List of regular expressions for match on
                    STDOUT, indicating failure.

                    Could contain app-cached precompiled 're'.

                stdoutok: List of regular expressions for match on
                    STDOUT, indicating success.

                    Could contain app-cached precompiled 're'.

                unicode:

                    True: see *re.UNICODE*, when set

                    False: default:=unset

        Returns:
            When successful returns 'True', else returns either 'False',
            or raises an exception.

        Raises:
            passed through exceptions:       

        """
        _ret = True
        _cnt = 0

        for k,v in predef.iteritems():

            if k=='setdefault': # set predefined default rule set
                # superposes reference values, reset changes in place
                self.stderrnok = self.rules_default['stderrnok']
                self.stderrok = self.rules_default['stderrok']
                self.stdoutnok = self.rules_default['stdoutnok']
                self.stdoutok = self.rules_default['stdoutok']
                self.reset()
                _cnt += 1

            elif k=='reset': # set predefined reset rule set
                self.reset()

            #
            # string match attributes
            #
            elif k=='multiline':
                self.cflags &= re.MULTILINE

            elif k=='ignorecase':
                self.cflags &= re.IGNORECASE

            elif k=='unicode':
                self.cflags &= re.UNICODE

            elif k=='dotall':
                self.cflags &= re.DOTALL

            elif k=='debug':
                self.cflags &= re.DEBUG

            #
            # weight of types
            #
            elif k=='priotype': # weight priority type,current 2 values
                if v in (True,) :
                    self.priotype = _P_OK
                elif v in (False,):
                    self.priotype = _P_NOK
                elif v.lower() in ('ok',) :
                    self.priotype = _P_OK
                else:
                    self.priotype = _P_NOK

            #
            # required number of matches - first matching counter dominates
            #
            elif k=='result': # sum of result counter
                self.result = v

            elif k=='resultok': # required number of OK for success
                self.resultok = v

            elif k=='resultnok': # required number of NOK for success
                self.resultnok = v

            #
            # mactch on: exit
            #
            elif k=='exitign':
                if v:
                    self.exittype = _E_IGN
                else:
                    if self.exittype == _E_IGN:
                        self.exittype = self.rules_default['exittype'] 
                self.exitign = v
                _cnt += 1
            elif k=='exittype':
                if v in (True,) :
                    self.exittype = _E_OK
                elif v in (False,) :
                    self.exittype = _E_NOK
                elif v.lower() in ('ok',) :
                    self.exittype = _E_OK
                elif v.lower() in ('value','val',) :
                    self.exittype = _E_VAL
                else:
                    self.exittype = _E_NOK
                _cnt += 1
            elif k=='exitval':
                self.exittype = _E_VAL
                self.exitval = int(v)
                _cnt += 1

            #
            # mactch on: STDERR
            #
            elif k=='stderrnok':
                self.stderrchk = True
                if not type(v) is list:
                    _v = [v]
                else:
                    _v = v
                for _ci in _v:
                    if _ci and type(_ci) in (str, unicode,):
                        _ci = re.compile(_ci,self.cflags)
                    if _ci:
                        self.stderrnok.append(_ci)
                _cnt += 1

            elif k=='stderrok':
                self.stderrchk = True
                if not type(v) is list:
                    _v = [v]
                else:
                    _v = v
                for _ci in _v:
                    if _ci and type(_ci) in (str, unicode,):
                        _ci = re.compile(_ci,self.cflags)
                    if _ci:
                        self.stderrok.append(_ci)
                _cnt += 1

            #
            # mactch on: STDOUT
            #
            elif k=='stdoutnok':
                self.stdoutchk = True
                if not type(v) is list:
                    _v = [v]
                else:
                    _v = v
                for _ci in _v:
                    if _ci and type(_ci) in (str, unicode,):
                        _ci = re.compile(_ci,self.cflags)
                    if _ci:
                        self.stdoutnok.append(_ci)
                _cnt += 1

            elif k=='stdoutok':
                self.stdoutchk = True
                if not type(v) is list:
                    _v = [v]
                else:
                    _v = v
                for _ci in _v:
                    if _ci and type(_ci) in (str, unicode,):
                        _ci = re.compile(_ci,self.cflags)
                    if _ci:
                        self.stdoutok.append(_ci)
                _cnt += 1

        if not _cnt:
            return False
        return _ret
    
    def apply(self,ret):
        """Apply rules on the input ret and weight the result.
        
        The provided match patterns and values are matched onto the input, and the 
        number of matches is counted.
        
        The number of required matches of sub-patterns has to be defined within the
        regexpr, e.g. by::

            'abv{4}[^v]+'

        this matches on::

            abvvvv

        Args:
            ret: Tuple received from 'SystemCalls'.
            
        Returns:
            When successful returns 'True', else returns either 'False',
            or raises an exception.

        Raises:
            passed through exceptions:
        
        """
        _result = self.default


        #-----------------------------
        # ***   filter and count   ***
        #-----------------------------

        #
        # filter string patterns on STDERR
        #
        if self.stderrchk: # patterns for STDERR are provided
            for _r in self.stderrok:
                _mx = ret[2]
                if type(_mx) == list:
                    _mx = '\n'.join(_mx)
                _m = _r.search(_mx,self.cflags)
                if type(_m) is NoneType:
                    continue
                self.stderrok_matched.append(_mx)
                self.stderrok_cnt += 1
                self.resultok_cnt += 1
            for _r in self.stderrnok:
                _mx = ret[2]
                if type(_mx) == list:
                    _mx = '\n'.join(_mx)
                _m = _r.search(_mx,self.cflags)
                if type(_m) is NoneType:
                    continue
                self.stderrnok_matched.append(_mx)
                self.stderrnok_cnt += 1
                self.resultnok_cnt += 1
        
        #
        # filter string patterns on STDOUT
        #
        if self.stdoutchk: # patterns for STDOUT are provided
            for _r in self.stdoutok:
                _mx = ret[1]
                if type(_mx) == list:
                    _mx = '\n'.join(_mx)
                _m = _r.search(_mx,self.cflags)
                if type(_m) is NoneType:
                    continue
                self.stdoutok_matched.append(_mx)
                self.stdoutok_cnt += 1
                self.resultok_cnt += 1
            for _r in self.stdoutnok:
                _mx = ret[1]
                if type(_mx) == list:
                    _mx = '\n'.join(_mx)
                _m = _r.search(_mx,self.cflags)
                if type(_m) is NoneType:
                    continue
                self.stdoutnok_matched.append(_mx)
                self.stdoutnok_cnt += 1
                self.resultnok_cnt += 1

        # sumup
        self.result_cnt = self.resultok_cnt + self.resultnok_cnt
        

        #
        # filter exit
        #
        # check exit value - store condition for later correlation with priorities
        # True: the requested exit value is matched
        # False: did not match the requested exit value
        #
        self._exitcond = True
        self._exitret = ret[0]
        if self.exittype == _E_IGN: # ignore exit values
            pass # exitconf == True
        elif self.exittype == _E_OK: # check whether current exit is OK 
            self._exitcond = ret[0] == 0
        elif self.exittype == _E_NOK: # check whether current exit is NOK
            self._exitcond = ret[0] > 0
        elif self.exittype == _E_VAL: # check whether present is predefined value
            self._exitcond = self.exitval == ret[0]


        #----------------------------------
        # ***   priotype and countres   ***
        #----------------------------------

        #
        # interpret the presence of specific partial results into overall resulting test status
        #
        
        #
        # any failure dominates
        #
        if self.priotype == _P_NOK: # ignore OK, when at least one NOK defined/failure occured
            if self.resultnok_cnt and self.resultnok_cnt >= self.resultnok: # threshold for expected failures
                _result = False
            elif self.resultok_cnt and self.resultok_cnt < self.resultok: # threshold for required success
                _result = False
            elif self.exitign: # no more failure criteria expected
                if self.stderrok and self.stderrok_cnt  == 0 or self.stdoutok and self.stdoutok_cnt == 0:
                    _result = False # no provided success criteria matched, thus define as failure 
                else: 
                    _result = True # no failure criteria was provided, thus define as success
            else:
                _result = self._exitcond


        #
        # any success dominates
        #
        elif self.priotype == _P_OK: # ignore NOK, when at least one OK defined
            if self.resultok_cnt and self.resultok_cnt >= self.resultok: # OK counter threshold match
                _result = True
            elif self.resultnok_cnt and self.resultnok_cnt < self.resultnok: # any is NOK
                _result = True            
            elif self.exitign: # no more success criteria expected
                if self.stderrok and self.stderrok_cnt or self.stdoutok and self.stdoutok_cnt:
                    _result = True 
                elif self.stderrnok and self.stderrnok_cnt == 0 or self.stdoutnok and not self.stdoutnok_cnt == 0:
                    _result = True
                else: 
                    _result = False

#                 if self.stderrnok and self.stderrnok_cnt or self.stdoutnok and self.stdoutnok_cnt:
#                     _result = False # no provided failure criteria matched, thus define as success 
#                 else: 
#                     _result = True # no failure criteria was provided, thus define as failure
            else:
                _result = self._exitcond

        #
        # weight the success and failures
        #
        elif self.priotype == _P_WEIGHT: # weight counters
            pass
        
        return  _result

    def states(self):
        """State values of current applied unittest.
        
        The applied parameters are available by 'repr()'.

        Args:
            ret: Tuple received from 'SystemCalls'.
                
        Returns:
            Returns the state of the last 'apply' operation.
            this is the result of the 'filter' and 'countres'
            described in the manual by the syntax-tree::

              r = {
                '_exitcond' = self._exitcond,   # resulting after applied self.exittype
                'exit' = self._exitret, # see _exitcond 
                'stderrnok' = self.stderrnok_matched,
                'stderrok' = self.stderrok_matched,
                'stdoutnok' = self.stdoutnok_matched,
                'stdoutok' = self.stdoutok_matched,

                'result' = self.result_cnt,
                'resultnok' = self.resultnok_cnt,
                'resultok' = self.resultok_cnt,
              }


        Raises:
            passed through exceptions:
        
          
        """
        _r = {}
        _r['_exitcond'] = self._exitcond
        _r['exit'] = self._exitret

        _r['stderrnok'] = self.stderrnok_matched
        _r['stderrok'] = self.stderrok_matched
        _r['stdoutnok'] = self.stdoutnok_matched
        _r['stdoutok'] = self.stdoutok_matched

        _r['result'] = self.result_cnt
        _r['resultnok'] = self.resultnok_cnt
        _r['resultok'] = self.resultok_cnt

        return _r
 
    def __str__(self):
        """Prints current rule set.
        """
        def _eenum(t):
            _e = _exitenums.get(t)
            if _e: return  _e+"("+str(t)+")"
            return str(t)

        def _penum(t):
            _e = _prioenums.get(t)
            if _e: return  _e+"("+str(t)+")"
            return str(t)

        ret = ""
        ret += "\ndefault       = "+str(self.default)
        ret += "\nexitign       = "+str(self.exitign)
        ret += "\nexittype      = "+str(_eenum(self.exittype))
        ret += "\nexitval       = "+str(self.exitval)
        ret += "\npriotype      = "+str(_penum(self.priotype))
        ret += "\nresult        = "+str(self.result)
        ret += "\nresultok      = "+str(self.resultok)
        ret += "\nresultnok     = "+str(self.resultnok)
        ret += "\nstderrchk     = "+str(self.stderrchk)
        ret += "\nstderrok      = "+str(self._re_pattern(self.stderrok))
        ret += "\nstderrnok     = "+str(self._re_pattern(self.stderrnok))
        ret += "\nstdoutchk     = "+str(self.stdoutchk)
        ret += "\nstdoutok      = "+str(self._re_pattern(self.stdoutok))
        ret += "\nstdoutnok     = "+str(self._re_pattern(self.stdoutnok))
        return ret

    def _re_pattern(self,pat):
        return [str(xi.pattern) for xi in pat]
        
    def __repr__(self):
        """Represents current rule set.
        """
        ret = "{"
        ret += "'default': "+str(self.default)
        ret += ", 'cflags': "+str(self.cflags)
        ret += ", 'multiline': "+str(self.multiline)
        ret += ", 'ignorecase': "+str(self.ignorecase)
        ret += ", 'unicode': "+str(self.unicode)
        ret += ", 'dotall': "+str(self.dotall)
        ret += ", 'debug': "+str(self.debug)
        ret += ", 'priotype': "+str(self.priotype)
        ret += ", 'result': "+str(self.result)
        ret += ", 'resultok': "+str(self.resultok)
        ret += ", 'resultnok': "+str(self.resultnok)
        ret += ", 'exitign': "+str(self.exitign)
        ret += ", 'exittype': "+str(self.exittype)
        ret += ", 'exitval': "+str(self.exitval)
        ret += ", 'stderrchk': "+str(self.stderrchk)
        ret += ", 'stderrnok': "+str(self.stderrnok)
        ret += ", 'stderrok': "+str(self.stderrok)
        ret += ", 'stdoutchk': "+str(self.stdoutchk)
        ret += ", 'stdoutnok': "+str(self.stdoutnok)
        ret += ", 'stdoutok': "+str(self.stdoutok)
        ret += "}"
        return ret

class SubprocessUnit(SystemCalls):
    """Wraps and checks results from execution of subprocesses by objects of class 'SProcUnitRules'.
    
    """
    def __init__(self,**kargs):
        """Prepares the caller interface for subprocess unit tests.
        
        Args:
            **kargs: Parameter specific for the operation,
                passed through to **setkargs**.

        Returns:
            When successful returns 'True'.

        Raises:
            passed through exceptions:
        
        """
        self.rules = None 

        super(SubprocessUnit,self).__init__(**kargs)
        
#         kargs['noparent'] = True
#         self.setkargs(**kargs)
#         kargs.pop('noparent')
    
    def apply(self,res=None):
        """Applies the linked rule set onto the res-data.

        Args:
            res: Result data to be filtered by a filter of type
                'epyunit.SProcUnitRules'.
            
        Returns:
            When a rule set is present and could be applied successful 
            returns 'True', else either 'False', or raises an exception.

        Raises:
            passed through exceptions:

        """
        if self.rules:
            return self.rules.apply(res)
        return False
    
    def displayit(self,ret,**kargs):
        """Displays result list ret in selected format.

        Args:
            ret: Data to be displayed.
            
            **kargs:

                outtarget: The target of display:
                
                    str: return as formatted string
                
                    stdout: print to sys.stdout
                
                    stderr: print to sys.stderr
                
                out: Output for display, valid:
                    
                    csv   : CSV with sparse records
                    
                    pass  : pass through STDOUT and STDERR from subprocess

                    repr  : Python 'repr()'
                    
                    str   : Python 'str()'
                    
                    xml   : XML
                
                default:=self.out

        Returns:
            When successful returns

                outtarget=='str': returns a printable formatted string

                outtarget in ('stdout', 'stderr',): 

                    'True':  for success

                    'False': for failure

        Raises:
            passed through exceptions:
        
        """
        
        return super(SubprocessUnit,self).displayit(ret,**kargs)

    def setkargs(self,**kargs):
        """Sets provided parameters for the subprocess call context.
        
        Applicable for the initial call of self.__init__(), 
        and later modification. Called for each start of
        a subprocess in order to update optional the specific 
        call context modification.

        Calls the parent method by default, could be supressed
        by option for the update of current objects class only.

        Args:
            **kargs: Parameters specific for the operations.

                noparent: Suppress call of parent class. 

                rules: Sets the rules object to be used.

        Returns:
            When successful returns 'True', else returns either 'False', or
            raises an exception.

        Raises:
            passed through exceptions:
            
        """
        _ret = True
        _args = kargs.copy()
        _noparent = _args.get('noparent',False)
        for k,v in kargs.iteritems():
            if k=='rules':
                self.rules = v
                _args.pop(k)
            elif k=='noparent':
                pass
        
        if not _noparent:
            _ret = super(SubprocessUnit,self).setkargs(**_args)
        
        if not self.rules:
            self.rules = SProcUnitRules(**kargs)
        if not self.rules:
            raise SubprocessUnitException("Failed to set rule set.")
        
        return _ret

#FIXME:

    def setruleset(self,ruleset):
        """Assigns the ruleset object as current.
        
        Args:
            ruleset: The ruleset to be applied as unittest.
                Replaces any previous.

        Returns:
            Returns the previous value of ruleset.
            If input is not an instance of 'SProcUnitRules'
            returns 'None'.

        Raises:
            passed through exceptions:
            
        """
        if not isinstance(ruleset,SProcUnitRules):
            return None
        _o = self.rules
        self.rules = ruleset
        return _o

    def getruleset(self):
        """Returns the current ruleset.
        
        Args:
            **kargs: Parameters passed transparently to
                 created SProcUnitRules instance.

                noparent: Suppress call of parent class. 

        Returns:
            Returns the current rules.

        Raises:
            passed through exceptions:
            
        """
        return self.rules

    def get_proceed(self,s=None): 
        """Verifies valid proceed type.

        Args:
            s: Requested type for validation.
            
        Returns:
            Supported types.

        Raises:
            passed through exceptions:
        
        """
        if s is NoneType:
            return 'subprocunits'
        if s in ('subprocunits',):
            return s
        else:
            return super(SubprocessUnit,self).get_proceed(s)

    def __str__(self):
        """Prints the current unit parameters including parent.
        """
        ret = super(SubprocessUnit,self).__str__()
        ret += "\n"
        ret += "\nSubprocessUnit.rules      = "+str(self.rules.__name__)
        return ret

    def __repr__(self):
        """Prints the current representation of unit parameters including parent and ruleset.
        """
        ret = super(SubprocessUnit,self).__repr__()
        ret = "{" + _parentrepr.sub(ur'\1', ret)
        ret += ", 'rules': "+self.rules.__class__.__name__
#        ret += ", 'rules': "+str(self.rules.__name__)
        ret += "}"
        return ret



# -*- coding: utf-8 -*-
"""Support for the automation of cross-process remote debugging with PyDev and Eclipse.

The module provides helpers for the debugging with the PyDev in the Eclipse IDE.
This includes utilities for the **cross-process debugging** by **Remote-Debugging**.
The PyDevERDbg module provides processes started outside the PyDev environment,
as well as processes under control of PyDev.

See 'http://pydev.org/manual_adv_remote_debugger.html'

The following parameters modify the control flow: 

* Module Variables:

    * **epyunit.PyDevERDbg.PYDEVD**: 
        Provides a pre-allocated controller object 
        for remote debugging by PyDev as Eclipse plugin. 
        Could be extended by custom instances as required.
        Just requires a simple import statement and could
        thereafter be controlled by parameters.

* Environment Variables:

    * **PYDEVDSCAN**: 
        The start directory for search on 'pydevd.py'. 
        If not set, the default is::
    
          px = $HOME/eclipse/eclipse
          px = os.path.abspath(px)
          px = os.path.realname(px)
          px = os.path.dirname(px)
          px = os.path.normpath(px)


"""
from __future__ import absolute_import
import pysourceinfo

__author__ = 'Arno-Can Uestuensoez'
__license__ = "Artistic-License-2.0 + Forced-Fairplay-Constraints"
__copyright__ = "Copyright (C) 2010-2016 Arno-Can Uestuensoez @Ingenieurbuero Arno-Can Uestuensoez"
__version__ = '0.1.11'
__uuid__='9de52399-7752-4633-9fdc-66c87a9200b8'

__docformat__ = "restructuredtext en"

import os,sys
version = '{0}.{1}'.format(*sys.version_info[:2])
if version < '2.7': # pragma: no cover
    raise Exception("Requires Python-2.7.* or higher")

from types import NoneType
import glob
#import termcolor
#import copy

from filesysobjects.FileSysObjects import findRelPathInSearchPath,clearPath,addPathToSearchPath

from epyunit.checkRDbg import _pderd_inTestMode_suppress_init, _dbg_self, _dbg_unit

PYDEVD = None



class PyDevERDbgException(Exception):
    pass

class PyDevERDbgLoadException(PyDevERDbgException):
    """Failed load of 'pydevd.py'. 
    """
    def __init__(self):
        ret="\n"
        ret+="\n"
        ret+="Cannot load module 'pydevd':\n"
        ret+="\n"
        ret+="   Set PYTHONPATH, or use more call parameters.\n"
        ret+="   See PyDev manual for path reference:\n"
        ret+="     'eclipse/plugins/org.python.pydev_x.x.x/pysrc/pydevd.py'\n" 
        ret+="\n"
        super(PyDevERDbgException,self).__init__(ret)

class PyDevERDbgServerException(PyDevERDbgException):
    """Debug server of PyDev is not running - cannot be reached. 
    """
    def __init__(self):
        ret="\n"
        ret+="\n"
        ret+="Cannot connect to debug server, start it using Eclipse menu:\n"
        ret+="\n"
        ret+="   In Debug-Perspective: 'PyDev -> Start Debug Server'\n"
        ret+="\n"
        super(PyDevERDbgException,self).__init__(ret)

class PyDevERDbg(object):
    """Provides automation for remote debugging of external Python process with PyDev.
    
    This particularly provides a prepared environment for 
    cross-process debugging by 'pydevd'.
    """
    
    _initok=False
    """Controls whether to be initialized."""       

    defaultargs={}
    
    dgbargs_default = {
        'host': 'localhost', # host(None)
        'stdoutToServer': True, # stdoutToServer(False)
        'stderrToServer': True, # stderrToServer(False) 
        'port': 5678, # port(5678) 
        'suspend': False, # suspend(True) : 
        'trace_only_current_thread': True, # trace_only_current_thread(False) 
        'overwrite_prev_trace': False, # 'ignore' == overwrite_prev_trace(False)
        'patch_multiprocessing': True, # patch_multiprocessing(False),
    }
    """
    Final default configuration arguments for a debug controller, see pydevd.settrace.
    """

    pydevd_glob = "org.python.pydev_[0-9]*.[0-9]*.[0-9]*/pysrc/pydevd.py"
    """
    Subpath glob pattern of PyDev plugin.
    """

    eclipse_glob = "eclipse/plugins/"+pydevd_glob
    """
    Subpath glob pattern for any eclipse based PyDev installation.
    """

    _clrargs = {'abs':True, 'non-existent':True,'split':True,'redundant':True,'shrink':True,'normpath':True}
    """
    Args for 'filesysobjects.clearPath'
    """

    def __init__(self,**kargs):
        """Create a control stub for a remote debugger.
        
        Search and load 'pydevd' for cross-process 
        remote debugging.
        
        Args:
            **kargs:
                label=<name>: An optional label for identifying the currrent instance.
                    The label could be provided as debugging flag.

                remotedebug: Switches remote debugging support On/Off.
                    Dependent of this parameter an internal 
                    call of startRemoteDebugExt is performed 
                    and a new instance initialized.
                
                rootforscan: Directory path for the module
                    'pydevd.py', else defaults of 
                    'scanEclipseForPydevd'.

                testflags: Flags to force specific behaviour
                    - mostly faulty - in order to test the module
                    itself. So, **do not use** these if you do not
                    know what these actually do. 
                     
                    These partially fail, but provide a sufficient
                    part of the control flow for the aspect of interest.
                     
                        ignorePydevd: Debugging the initial bootstrap
                            of an simulated external process. Ignores 
                            present loaded debug support. For debugging
                            of the debug support.

                        ignorePydevdSysPath: Debugging the initial
                            bootstrap of an simulated external process.
                            Ignores load by current 'sys.path'. For debugging
                            of the debug support.

                        ignorePydevdCallParam: Debugging the initial
                            bootstrap of an simulated external process.
                            Ignores current parameter for load. For debugging 
                            of the debug support.

        Returns:
            Creates a proxy instance.

        Raises:
            passed through exceptions:
        """
        self._itfp = False
        self._itfs = False
        self._itfcp = False
        for k,v in kargs.items():
            if k == 'testflags':
                for tf in v.split(','):
                    if tf == 'ignorePydevd':
                        self._itfp = True
                    elif tf == 'ignorePydevdSysPath':
                        self._itfs = True
                    elif tf == 'ignorePydevdCallParam':
                        self._itfcp = True
            elif k == 'label':
                self.label = v

        # check whether running in pydevd.py
        if not self._itfp and sys.modules.get('pydevd'): # already loaded
            self.runningInPyDevDbg = True
            self.pydevdpath = sys.modules.get('pydevd')
            self.setDebugParams(**kargs)
        else: # has to be loaded
            self.pydevdpath = self.scanEclipseForPydevd(kargs.get('rootforscan'),**kargs)
            if self.pydevdpath:
                addPathToSearchPath(os.path.dirname(self.pydevdpath),**{'exist':True,'prepend':True,}) 
                try:
                    import pydevd #@UnresolvedImport #@UnusedImport
                except Exception as e:
                    raise PyDevERDbgException("Load of 'pydevd' failed:"+str(os.path.dirname(self.pydevdpath))+"\n"+str(e))
                self.runningInPyDevDbg = True
            else:
                self.runningInPyDevDbg = False

            self.setDebugParams(**kargs)
        
        # starts remote debug immediately
        self.remotedebug = kargs.get("remotedebug",False)
        if self.remotedebug:
            self.startDebug()
        self._initok=True

    def setDebugParams(self,**kargs):
        """Sets the parameters for debug. 
               
        Args:
            **kargs:
                host:
                
                    Hostname where the debug server is running.
    
                    From **pdevd.py**:
                    The user may specify another host, if the debug server 
                    is not in the same machine (default is the local host).

                ignore:
                
                    When set to False success is mandatory, else
                    an exception it raised.
                    Set this to True/default, when in production 
                    systems.

                port:
                
                    Port the debug server is listening on.
                    From **pdevd.py**:
                    Specifies which port to use for communicating with the
                    server (note that the server must be started in the 
                    same port). 
                    **Note**: currently it's hard-coded at 5678 
                    in the client

                pydevdpath:
                
                    Required path pointing to directory for 
                    source of pydevd in PyDev subdirectory tree.

                remotedebug:
                
                    Switches remote debugging support On/Off.

                stderrToServer:
                
                    Sets whether stderr is directed to debugserver.
                    From **pdevd.py**:
                    When this is true, the stderr is passed to the debug server
                    so that they are printed in its console and not in this 
                    process console.

                stdoutToServer:
                
                    Sets whether stdout is directed to debugserver.
                    From **pdevd.py**:
                    When this is true, the stdout is passed to the debug server.

                suspend:
                
                    If set to True, stops immediately after settrace() call,
                    else at next valid break-condition.
                    From **pdevd.py**:
                    Whether a breakpoint should be emulated as soon as 
                    this function is called.

                trace_only_current_thread:
                
                    From **pdevd.py**:
                    Determines if only the current thread will be traced or all
                    future threads will also have the tracing enabled.

        Returns:
            The location of pydevd.py
    
        Raises:
            AttributeError:

        """
        self.dbgargs = self.dgbargs_default
        for k,v in kargs.iteritems():
            if k=='pydevdpath':
                pydevdpath=v
            elif k=='ignore':
                ignore=v 
            elif k=='remotedebug':
                remotedebug=v

            elif k=='host':
                self.dbgargs['host'] = v
            elif k=='stdoutToServer':
                self.dbgargs['stdoutToServer'] = v
            elif k=='stderrToServer':
                self.dbgargs['stderrToServer'] = v
            elif k=='port':
                self.dbgargs['port'] = v
            elif k=='suspend':
                self.dbgargs['suspend'] = v
            elif k=='trace_only_current_thread':
                self.dbgargs['trace_only_current_thread'] = v
            elif k=='overwrite_prev_trace':
                self.dbgargs['overwrite_prev_trace'] = v
            elif k=='patch_multiprocessing':
                self.dbgargs['patch_multiprocessing'] = v

    def scanEclipseForPydevd(self,rootforscan=None, **kargs):    
        """Scans filesystem directory tree of Eclipse for PyDev plugin containing 'pydevd'. 

        Scans for 'pydevd' required for subprocess debugging
        by the Debug-Server of PyDev. See PyDev manual for 
        path reference, the default pattern is:
            ::

                eclipse/plugins/org.python.pydev_x.x.x/pysrc/pydevd.py

        The path is the containment path of the file *pydevd.py* and has to be 
        included in the *sys.path* variable for activation.
        
        The *scanEclipseForPydevd* method performs a search and returns the 
        path in case of a match.
         
        Searches in following priority:
        
            1. call parameter
                
            2. environment PYDEVDSCAN

                Contains either a hook for the pydevd_glob pattern,
                or an FQPN to the appropriate 'pydevd.py'.
            
            3. sys.path
                
            4. <HOME>/eclipse/eclipse
            
                Where the path is a symbolic link to the executable,
                the realpath will be evaluated from the link.

            5. search install directory when installed within the
                'dropins' directory::

                os.path.dirname(__file__)+""

        The first match of containing directory for 'pydevd.py'
        is returned, thus ambiguity in case of multiple occurrences
        has to be avoided.

        Args:
            rootforscan: Start directory for tree-scan, either a
                single, multiple in PATH notation.
    
        **kargs:
            altpat=(<literal>|<glob>): Alternative pattern, varies 'eclipse_glob',
                and 'pydevd_glob'.

            strict: Provided parameter has to match, else error.

                Default is to try all, when supplied params do not
                match default values are checked.

            version=(a,b,c): Provide version to be requested. It is recommended
                to combine this with 'strict'.
    
                version=(a,b,c)
                a := [0-9]+
                b := [0-9]+
                b := [0-9]+
    
        Returns:
            The location of pydevd.py
    
        Raises:
            AttributeError:

        """
        _mlist = [] # list of matches

        _apat = kargs.get('altpat')
        _vers = kargs.get('version')
        _pg = self.pydevd_glob
        if _apat and _vers:
            raise PyDevERDbgLoadException("One only supported (version, altpat)=('"+str(_vers)+"', '"+str(_apat)+"')")
        elif _vers:
            _pg = "org.python.pydev_"+_vers[0]+"."+_vers[1]+"."+_vers[2]+"*/pysrc/pydevd.py"
            _apat = "eclipse/plugins/"+_pg
        elif not _apat:
            _apat = self.eclipse_glob

        _st = kargs.get('strict',False)
        
        if rootforscan == NoneType:
            _rfs = []
        elif type(rootforscan) != list:
            _rfs = [rootforscan]
            clearPath(_rfs,**self._clrargs)
        else:
            _rfs = rootforscan[:]
            clearPath(_rfs,**self._clrargs)

        p = None
        
        # 1. call parameter
        if not self._itfcp:
            p = findRelPathInSearchPath(_apat,_rfs)
            if p:
                return p
            elif _rfs and _st:
                return

        # 2. environment PYDEVDSCAN
        if not p:
            try:
                _rfs = os.environ['PYDEVDSCAN']
                clearPath(_rfs,**self._clrargs)
                p = findRelPathInSearchPath(_apat,_rfs)
                if p:
                    return p
                elif _rfs and _st:
                    return
            except:
                pass

        #
        # following are the preset and hard-coded defaults
        #
        
        # 3. search 'pydevd.p' by sys.path
        if not self._itfs and not rootforscan:
            p = findRelPathInSearchPath('pydevd.py',sys.path)
            if p:
                return p

        # 4. <HOME>/eclipse/eclipse
        if not p:
            _r = os.environ['HOME']
            _r += os.sep+'eclipse'+os.sep+'eclipse'
            if os.path.exists(_r):
                if kargs.get('altpat'):
                    _apat = kargs.get('altpat')
                    if _apat.startswith('eclipse/plugins/'):
                        _apat = _apat[7:]
                else:
                    _apat = self.pydevd_glob
                _r = os.path.normpath(os.path.dirname(os.path.realpath(_r))+"/plugins/")

            p = findRelPathInSearchPath(_apat,[_r])
            if p:
                return p

        # 5. search install directory when installed within the 'dropins' directory::
        if not p:
            if os.path.exists(os.path.normpath("../plugins")) and os.path.exists(os.path.normpath("../eclipse")):
                _r = glob.glob(os.path.normpath("../plugins/"+_pg))
            p = findRelPathInSearchPath(_apat,_r)
            if p:
                return p

    def startDebug(self):
        """Starts remote debugging for PyDev.
        """        
        # already loaded
        if self.runningInPyDevDbg:
            try:
                pydevd.settrace(
                    host=self.dbgargs['host'],
                    port=self.dbgargs['port'],
                    stdoutToServer=self.dbgargs['stdoutToServer'],
                    stderrToServer=self.dbgargs['stderrToServer'],
                    suspend=self.dbgargs['suspend'],
                    trace_only_current_thread=self.dbgargs['trace_only_current_thread']
                    )
                #OK-ref: pydevd.settrace(host=None,port=5678,stdoutToServer=False,stderrToServer=False,suspend=False,trace_only_current_thread=True)
            except Exception as e:
                raise PyDevERDbgException(e)
        else:
            raise PyDevERDbgException("ERROR:Requires init:self.runningInPyDevDbg="+str(self.runningInPyDevDbg))
        if _dbg_self or _dbg_unit:
            print >>sys.stderr,"RDBG:debug started"
            
    def stopDebug(self):
        """Stops remote debugging for PyDev.
        """
        try:
            if self.runningInPyDevDbg:
                pydevd.stopdebug()
        except Exception as e:
            raise PyDevERDbgException()

    def setFork(self):
        """Prepares debugging after fork.
        """
        if self.runningInPyDevDbg:
            pydevd.settrace_forked()
        pass

    def __str__(self):
        """Prints current remote debug parameters.
        """
        ret  = ""
        ret += "\nPyDevERDbg.host                      = "+str(self.host)
        ret += "\nPyDevERDbg.stdoutToServer            = "+str(self.stdoutToServer)
        ret += "\nPyDevERDbg.stderrToServer            = "+str(self.stderrToServer)
        ret += "\nPyDevERDbg.port                      = "+str(self.port)
        ret += "\nPyDevERDbg.suspend                   = "+str(self.suspend)
        ret += "\nPyDevERDbg.trace_only_current_thread = "+str(self.trace_only_current_thread)
        ret += "\nPyDevERDbg.overwrite_prev_trace      = "+str(self.overwrite_prev_trace)
        ret += "\nPyDevERDbg.patch_multiprocessing     = "+str(self.patch_multiprocessing)
        return ret

    def __repr__(self):
        """Prints the current representation of remote debug parameters.
        """
        ret = "{"
        ret += "'host': "+str(self.host)
        ret += ", 'stdoutToServer': "+str(self.stdoutToServer)
        ret += ", 'stderrToServer': "+str(self.stderrToServer)
        ret += ", 'port': "+str(self.port)
        ret += ", 'suspend': "+str(self.suspend)
        ret += ", 'trace_only_current_thread': "+str(self.trace_only_current_thread)
        ret += ", 'overwrite_prev_trace': "+str(self.overwrite_prev_trace)
        ret += ", 'patch_multiprocessing': "+str(self.patch_multiprocessing)
        ret += "}"
        return ret


if not _pderd_inTestMode_suppress_init:
    if _dbg_self or _dbg_unit:
        print >>sys.stderr,"RDBG:init PYDEVD"
    if not PYDEVD or not PyDevERDbg._initok:
        PYDEVD=PyDevERDbg()
        _pydevd = PYDEVD.scanEclipseForPydevd()
        if _dbg_self:
            print >>sys.stderr,"RDBG:found pydevd.py:"+str(_pydevd)
        if _dbg_unit:
            print >>sys.stderr,"RDBG:found pydevd.py"
        addPathToSearchPath(os.path.dirname(_pydevd),**{'exist':True,'prepend':True,}) 
        import pydevd #@UnresolvedImport
        pass
pass



# -*- coding: utf-8 -*-
"""The package 'epyunit' extends the standard PyUnit package for blackbox tests on arbitrary executables.

"""
from __future__ import absolute_import

__author__ = 'Arno-Can Uestuensoez'
__license__ = "Artistic-License-2.0 + Forced-Fairplay-Constraints"
__copyright__ = "Copyright (C) 2010-2016 Arno-Can Uestuensoez @Ingenieurbuero Arno-Can Uestuensoez"
__version__ = '0.1.12'
__uuid__='9de52399-7752-4633-9fdc-66c87a9200b8'

__docformat__ = "restructuredtext en"

__package__ = 'epyunit'
__all__=[
    "SystemCalls",
    "SubprocessUnit",
    "SProcUnitRules",
    "selftest",
    "checkRDbg",
    "PyDevERDbg",
    "TestExecutable",
]



# -*- coding: utf-8 -*-
"""Initialize PyDevERDbg.
Manages command line parameters. Extracts rdbg-options and provides
values for the main module 'epyunit.PyDevERDbg'.

"""
from __future__ import absolute_import

__author__ = 'Arno-Can Uestuensoez'
__license__ = "Artistic-License-2.0 + Forced-Fairplay-Constraints"
__copyright__ = "Copyright (C) 2010-2016 Arno-Can Uestuensoez @Ingenieurbuero Arno-Can Uestuensoez"
__version__ = '0.1.12'
__uuid__='9de52399-7752-4633-9fdc-66c87a9200b8'

__docformat__ = "restructuredtext en"

import sys,os
version = '{0}.{1}'.format(*sys.version_info[:2])
if version < '2.7': # pragma: no cover
    raise Exception("Requires Python-2.7.* or higher")

_pderd_inTestMode_suppress_init = False
"""Forces test mode, enables the setting of partially erroneous parameters. 
"""
_dbg_self = False
"""Debugs the debugging.
"""
_dbg_unit = False
"""Debugs units.
"""

_rdbgfwd_default = 0
"""Control of forwarding the debugging enabled state.
Possible values are:

    hopcnt: The nested subprocesses to be debugged.

        hopcnt >=0 : The number of nested subprocess 
            hops to be debugged.
    
        hopcnt <0 : The number of nested subprocess 
            hops NOT to be debugged.

    'all': Debug all nested levels of subprocesses.

    label: Activate remote debug for the process with the previously
        assigned label only.

    label = [label-lst]: List of labels.

"""

_rdbg_default = "localhost:5678" 
"""The default values for the peer RemoteDebugServer as defined by PyDev.
"""

def checkAndInitRDbg(**kargs):
    """Checks input options from sys.argv and returns resulting debug parameters.
    The following options are checked and removed from sys.argv, thus has to be
    added by the caller application again  when required for following nested calls:

        --rdbg:

            Enables local debugging by default, provides optionally altered
            connection parameters for the RemoteDebugServer.
        
        --rdbg-forward:

            Defines forwarding of debugging state, either by passing
            the current instance, or by additionally debugging the next level(s)
            of nested subprocesses. 

    Test options and flags:

        --pderd_inTestMode_suppress_init:

            Control initialization of the preconfigured debug stub.
            It is foreseen to be the only instance under normal
            circumstances.

        --pderd_debug_self:

            Enabled debugging messages for debug, this also includes
            the pre-debug initialization of the remote debug server.

        --pderd_unit_self:

            Enabled log messages for unittests.    

    Args:
        callstr: a prepared shell-style call.
        
        **kargs:

            label: The label identifying the current process, default:=callname.

            fpname: The file path name of the main file for the current process, default:=callname.

            rdbg: Same as '--rdbg', for additional processing see 'noargv'.
        
            rdbgforward: Same as '--rdbg-forward', for additional processing see 'noargv'.

            noargv: Suppresses argv processing completely for the previous 
            arguments. The testflags are still processed.

    Returns:
        When successful returns tuple of the following debug parameters.
        
        return/value := ( rdbgthis, rdbg, rdbgfwd, )

            rdbgthis = (True|False): Debugging of current process/thread is enabled.

            rdbg = [host][:port]: Connection parameter of peer RemoteDebugServer
            
            rdbgfwd = <partialprocessed>: Forward debug parameters

                partialprocessed: One of:

                    hop = hopcnt - 1

                    'all'

                    label

    Raises:
    
    """
    global _pderd_inTestMode_suppress_init
    global _dbg_self
    global _dbg_unit
    
    _noarg = False
    _rdbgthis = False
    _rdbg = None
    _rdbgfwd = None

    _lbl = None
    _fpname = None
    
    for k,v in kargs.items():
        if k == 'label':
            _lbl = v
        
        elif k == 'fpname':
            _fpname = v

        elif k == 'rdbg':
            _rdbg = v
            
        elif k == 'rdbgforward':
            _rdbgfwd = v

        elif k == 'noargv':
            _noarg = v
            
        else:
            raise("Unknown option:"+str(k))

    if not _lbl:
        _lbl = os.path.basename(sys.argv[0])
    if not _fpname:
        _fpname = os.path.abspath(sys.argv[0])

    # prevents init, for test only
    if '--pderd_inTestMode_suppress_init' in sys.argv:
        _pderd_inTestMode_suppress_init = True
        for px in sys.argv:
            sys.argv.pop(sys.argv.index(px))
    else:
        _pderd_inTestMode_suppress_init = False

    # activates self debug
    if '--pderd_debug_self' in sys.argv:
        _dbg_self = True
        sys.argv.pop(sys.argv.index('--pderd_debug_self'))
    else:
        _dbg_self = False

    # activates self unittests
    if '--pderd_unit_self' in sys.argv:
        _dbg_unit = True
        sys.argv.pop(sys.argv.index('--pderd_unit_self'))
    else:
        _dbg_unit = False
    
    if not _noarg and "--rdbg" in sys.argv:
        _ai = sys.argv.index("--rdbg")
        _a = sys.argv.pop(_ai) #--rdbg
        if _ai < len(sys.argv) and sys.argv[_ai][:1] != '-':
            _rdbg = sys.argv.pop(_ai) #value
        else:
            _rdbg = _rdbg_default

    if not _noarg and "--rdbg-forward" in sys.argv:
        _ai = sys.argv.index("--rdbg-forward")
        _a = sys.argv.pop(_ai) #--rdbg
        if _ai < len(sys.argv) and sys.argv[_ai][:1] != '-':
            _rdbgfwd = sys.argv.pop(_ai) #value
        else:
            _rdbgfwd = _rdbgfwd_default

    # activates connection to remote debug server
    if _rdbg:
        _rdbgthis = True # here for now active, could be reset when forwarding only, see following

        # forwards debugging state to nested subprocesses
        if _rdbgfwd:
            if _rdbgfwd.lower() == 'all': # all
                _rdbgthis = True

            elif type(_rdbgfwd) == int and _rdbgfwd: # hopcount
                if _rdbgfwd >= 0: # debug the first:  [0:hopcnt]
                    _rdbgfwd -= 1
                    if _rdbgfwd >= 0:
                        _rdbgthis = True
                    else:
                        _rdbgthis = False

                else: # debug from hopcnt on: [hopcnt:]
                    _rdbgfwd += 1
                    if _rdbgfwd < 0:
                        _rdbgthis = False
                    else:
                        _rdbgthis = True

            elif type(_fpname) is str and os.path.exists(_fpname): # fpname
                if os.path.abspath(_fpname) == os.path.abspath(sys.argv[0]): 
                    _rdbgthis = True
                else:
                    _rdbgthis = False

            elif type(_rdbgfwd) is str: # label
                if _rdbgfwd == _lbl:
                    _rdbgthis = True

            else:
                _rdbgthis = False

    return (_rdbgthis,_rdbg,_rdbgfwd,)
